#/opt/local/bin/python
# make plot easily

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")

import locale
import numpy             	as np
import matplotlib.pyplot 	as plt
from   numpy             	import arange
from   read_mhd                	import maxmin_ran
from   run_info          	import tick_info,mrplt_info
from   matplotlib        	import rc
from   pkg_mhdplot             	import mhd_vpkg
from   openggcm_info     	import mhdtime,read_swdata,mhdt2hrmi,runinfo
from   matplotlib.colors       	import LogNorm
from   matplotlib.ticker       	import MultipleLocator, FuncFormatter
#from   mpl_toolkits.axes_grid1 	import make_axes_locatable


def colorful_text(vstr,col):
  rc('text',usetex=True)
  rc('text.latex', preamble='\\usepackage{color}')
  cstr=' '
  for i in range(len(vstr)) :
    cstr=cstr+r'\textcolor{'+col[i]+'}{'+vstr[i]+'} '
# print cstr
  return cstr



def easyplot(FPS,tt,val,vran,vstr,title,tevent='none'):
  "plot values easily                                                    \n \
   FPS : example) $DIR/$RUN.**.**   					\
         $RUN (the OpenGGCM runname) should be located after $DIR/.	\
   tstr	\
   tstr = val=[ pcpot,parea,marea,drate,nrate,mrdif,dmfds,dm1,dm2                ]        "


#### set tick information depending on the events

  info=runinfo(FPS)
  tstr=mhdtime(info.ti)
  dtmaj,dtmin,dtlab,xran= tick_info(info.run)

  if xran[0]==-9999 and xran[-1]==9999 :
    xran    =  [ tt[-1][0], tt[-1][-1] ]


  i       =  0
  xtick1  =  arange(xran[0],xran[1]+dtmaj,dtmaj)   # major tick location
  xtick2  =  arange(xran[0],xran[1]+dtmin,dtmin)   # minor tick location
  xlabel  =  mhdt2hrmi(xtick1,tstr,dtlab)         # create tick label
  ytick1  =  arange(0,30,6)
  ytick2  =  arange(0,25,1)



#### mr_rate graph

  fig=plt.figure(figsize=(8,11))


  for i in range(len(val)):

    ax1 = plt.subplot(len(val),1,i+1)
    plt.subplots_adjust(hspace = .001)


    print(len(val[i]),type(val[i][-1]))
    if type(val[i][-1]) is list  :      # draw several values in the same plot

      if val[i][0]==0 :                 # share the plot
        for jj in val[i][1:] : pl1 = ax1.plot(tt[i],jj)
        ax1.set_ylim (vran[i])

      if val[i][0]==1 :                 # share only xaxis
                                        # ONLY TWO VALUES should be used!!
        pl1 = ax1.plot(tt[i],val[i][1])
        for tl in ax1.get_yticklabels(): tl.set_color('b')
        ax1.set_ylim (vran[i][0])

        ax2 = ax1.twinx()
        pl2 = ax2.plot(tt[i],val[i][2],'g')
        for tl in ax2.get_yticklabels(): tl.set_color('g')
        ax2.set_ylim (vran[i][1])

#       for j in range(len(tt[i])):
#          if tt[i][j]>=12000 and tt[i][j]<=13500:
#              print tt[i][j],val[i][1][j],val[i][2][j]

    else :                              # draw one value plot

      print(vran[i])
      pl1=ax1.plot(tt[i],val[i])
      ax1.set_ylim (vran[i])

    plt.setp(ax1.get_yticklabels()[-1],visible=False)
    ax1.text(0.01,0.9,vstr[i],ha='left',va='top', \
             fontsize=10,transform=ax1.transAxes)


    ax1.set_xlim   (xran)
    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)


    if i==len(tt)-1:
      ax1.set_xticklabels (xlabel)
    else :
      ax1.set_xticklabels ([ '' for j in xlabel])

    ax1.grid(axis='x')
    ax1.axhline(y=0,ls=':',c='k')

    if tevent!='none':
      ax1.axvline(x=tevent,ls='-',lw=2,c='k')

    if i==0 :
      ax1.set_title(title,fontsize=12)

# plt.tight_layout(h_pad=0.2)
# plt.show()
  plt.savefig(FPS)
  plt.close()


  return



def easy1Dplot(fps,tt,vv,ty,vline,ylab,vran,vstr,title,tevent='none',tkinfo=[],figsize=(8,11)):
  " time  info  : tt   = [ [t1,t2], [t2], [t3,t1,t4]                            \
    value info  : vv   = [ [v1,v2], [v2], [v3,v1,v4]                            \
    line style  : ty   = [ ['ro','b-'], ['r-'], ['r-','b-','g-]                 \
    string of vv: vstr = [ 'IMF By & Bz [nT]', 'Vsw [km/s]','CPCP [kV]' ]       \
    yrange of vv: vran = [ [-10,10], [300,500],[0,300] ]                        \
    title of fps: title= 'Jan 10 1997 summary plot'                             \
    eventtime for vertical bar :     tevent = 20000                             \
    tevent[0]   : must be the event time, makes thick black vertical lines      \
    tevent[1:]  : other timelines, makes thin red vertical lines                "

  print('easy1Dplot starts!')

  if len(tkinfo)!=0 :
    tstr = mhdtime(tkinfo[0])
    dtmaj,dtmin,dtlab,xran = tkinfo[1:]
  else :
    info = runinfo(fps)
    tstr = mhdtime(info.ti)
    dtmaj,dtmin,dtlab,xran = tick_info(info.run)

  if xran[0]==-9999 and xran[-1]==9999 :
    xran = [ tt[-1][0][0], tt[-1][0][-1] ]

  print(dtmaj,dtmin,dtlab,xran)

  i       =  0
  xtick1  =  arange(xran[0],xran[1]+dtmaj,dtmaj)   # major tick location
  xtick2  =  arange(xran[0],xran[1]+dtmin,dtmin)   # minor tick location
  xlabel  =  mhdt2hrmi(xtick1,tstr,dtlab)          # create tick label

# fig=plt.figure(figsize=(8,11))
  if figsize==() :fig=plt.figure()
  else           :fig=plt.figure(figsize=figsize)

  for i in range(len(tt)):

    ax1=plt.subplot(len(tt),1,i+1)
    plt.subplots_adjust(hspace = .001)

    for j in range(len(tt[i])):
#      print(i,j)

      lst = ty[i][j].split(':')
#      breakpoint()
      if len(lst)==1: ax1.plot(tt[i][j],vv[i][j],ty[i][j])
      else:
#         print(lst[1],tt[i][j],vv[i][j])
         if lst[1]=='plot': ax1.plot(tt[i][j],vv[i][j],lst[0])
         if lst[1]=='step': ax1.step(tt[i][j],vv[i][j],lst[0],where='mid')
         if lst[1]=='yerr': ax1.errorbar(tt[i][j],vv[i][j][0],yerr=vv[i][j][1],fmt=lst[0])


#   ax1.text(0.01,0.9,vstr[i],ha='left',va='top', \
#            fontsize=10,transform=ax1.transAxes)

    if isinstance(vstr[i],list):
      vstr1,colr1,x1,y1 = vstr[i]
      for j in range(len(vstr1)):
         ax1.text(x1[j],y1[j],vstr1[j],ha='left',va='top',color=colr1[j],	\
             fontsize=10,transform=ax1.transAxes)
    else :
      ax1.text(0.99,0.9,vstr[i],ha='right',va='top', \
               fontsize=10,transform=ax1.transAxes)

#...xticks
    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)
    if i==len(tt)-1: ax1.set_xticklabels (xlabel,fontsize=10);ax1.set_xlabel('TIME [UT]')
    else           : ax1.set_xticklabels ([ '' for j in xlabel])
    ax1.set_xlim   (xran)
    ax1.tick_params(axis='x',which='major',direction='inout',length=6) #,width=2)
    ax1.tick_params(axis='x',which='minor',direction='inout',length=6)


#...yticks
    ax1.set_ylabel(ylab[i],fontsize=10)
    if isinstance(vran[i],list):
        ax1.set_ylim(vran[i][0],vran[i][1])
        if len(vran[i])>=3:
          ytick1 = arange(vran[i][0],vran[i][1]+vran[i][2],vran[i][2])
          ax1.set_yticks (ytick1)
          if len(vran[i])==4:
            ytick2 = arange(vran[i][0],vran[i][1],vran[i][3])
            ax1.set_yticks (ytick2,minor=True)
    plt.setp(ax1.get_yticklabels(),fontsize=10)
    plt.setp(ax1.get_yticklabels()[-1],visible=False)
    plt.setp(ax1.get_yticklabels()[0],visible=False)

    ax1.grid(axis='both')
    if isinstance(vran[i],list):
        if vran[i][0]==0: pass
        else: ax1.axhline(y=0,ls=':',c='k')
    else:   ax1.axhline(y=0,ls=':',c='k')

    if tevent!='none'     : ax1.axvline(x=tevent,ls='--',lw=2,c='k')
    if len(vline[i])!=0   :
       if   vline[i][-1] == 'v' :
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],ls=j[1],lw=j[2],c=j[3])
       elif vline[i][-1] == 'h' :
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],ls=j[1],lw=j[2],c=j[3]);print(j)
       elif vline[i][-1] == 'vp':
         for j in vline[i][:-1]  : ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4,linewidth=2.,linestyle='--')
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2]);ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'hp':
         for j in vline[i][:-1]  : ax1.axhspan(j[0],j[1],facecolor=j[2],alpha=0.4)
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],lw=2.,ls='--',c=j[2]);ax1.axhline(y=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'vph':
         for j in vline[i][:-1]  :
            ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4)
            ax1.axhline(y=j[3],ls=j[4],lw=j[5],c=j[6])
            ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2])
            ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])



    if i==0 :
      ax1.set_title(title,fontsize=12)

# plt.show()
# plt.tight_layout(h_pad=0.2)
  print("Save ",fps)
  plt.savefig(fps)
  plt.savefig(fps[:-4]+'.png')
  plt.close()

  return

def easy1Dplot_dot(fps,tt,vv,ty,vline,ylab,vran,vstr,title,tevent='none',tkinfo=[],figsize=(8,11)):
  " time  info  : tt   = [ [t1,t2], [t2], [t3,t1,t4]                            \
    value info  : vv   = [ [v1,v2], [v2], [v3,v1,v4]                            \
    line style  : ty   = [ ['ro','b-'], ['r-'], ['r-','b-','g-]                 \
    string of vv: vstr = [ 'IMF By & Bz [nT]', 'Vsw [km/s]','CPCP [kV]' ]       \
    yrange of vv: vran = [ [-10,10], [300,500],[0,300] ]                        \
    title of fps: title= 'Jan 10 1997 summary plot'                             \
    eventtime for vertical bar :     tevent = 20000                             \
    tevent[0]   : must be the event time, makes thick black vertical lines      \
    tevent[1:]  : other timelines, makes thin red vertical lines                "

  print('easy1Dplot_dot starts!')

  if len(tkinfo)!=0 :
    tstr = mhdtime(tkinfo[0])
    dtmaj,dtmin,dtlab,xran = tkinfo[1:]
  else :
    info = runinfo(fps)
    tstr = mhdtime(info.ti)
    dtmaj,dtmin,dtlab,xran = tick_info(info.run)

  if xran[0]==-9999 and xran[-1]==9999 :
    xran = [ tt[-1][0][0], tt[-1][0][-1] ]

  print(dtmaj,dtmin,dtlab,xran)

  i       =  0
  xtick1  =  arange(xran[0],xran[1]+dtmaj,dtmaj)   # major tick location
  xtick2  =  arange(xran[0],xran[1]+dtmin,dtmin)   # minor tick location
  xlabel  =  mhdt2hrmi(xtick1,tstr,dtlab)          # create tick label

# fig=plt.figure(figsize=(8,11))
  if figsize==() :fig=plt.figure()
  else           :fig=plt.figure(figsize=figsize)

  for i in range(len(tt)):

    ax1=plt.subplot(len(tt),1,i+1)
    plt.subplots_adjust(hspace = .001)

    for j in range(len(tt[i])):
#      print(i,j)

      lst = ty[i][j].split(':')
#      breakpoint()
      if len(lst)==1: ax1.plot(tt[i][j],vv[i][j],ty[i][j],ls='',marker='.',ms=4)
      else:
#         print(lst[1],tt[i][j],vv[i][j])
         if lst[1]=='plot': ax1.plot(tt[i][j],vv[i][j],lst[0])
         if lst[1]=='step': ax1.step(tt[i][j],vv[i][j],lst[0],where='mid')
         if lst[1]=='yerr': ax1.errorbar(tt[i][j],vv[i][j][0],yerr=vv[i][j][1],fmt=lst[0])


#   ax1.text(0.01,0.9,vstr[i],ha='left',va='top', \
#            fontsize=10,transform=ax1.transAxes)

    if isinstance(vstr[i],list):
      vstr1,colr1,x1,y1 = vstr[i]
      for j in range(len(vstr1)):
         ax1.text(x1[j],y1[j],vstr1[j],ha='left',va='top',color=colr1[j],	\
             fontsize=10,transform=ax1.transAxes)
    else :
      ax1.text(0.99,0.9,vstr[i],ha='right',va='top', \
               fontsize=10,transform=ax1.transAxes)

#...xticks
    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)
    if i==len(tt)-1: ax1.set_xticklabels (xlabel,fontsize=10);ax1.set_xlabel('TIME [UT]')
    else           : ax1.set_xticklabels ([ '' for j in xlabel])
    ax1.set_xlim   (xran)
    ax1.tick_params(axis='x',which='major',direction='inout',length=6) #,width=2)
    ax1.tick_params(axis='x',which='minor',direction='inout',length=6)


#...yticks
    ax1.set_ylabel(ylab[i],fontsize=10)
    if isinstance(vran[i],list):
        ax1.set_ylim(vran[i][0],vran[i][1])
        if len(vran[i])>=3:
          ytick1 = arange(vran[i][0],vran[i][1]+vran[i][2],vran[i][2])
          ax1.set_yticks (ytick1)
          if len(vran[i])==4:
            ytick2 = arange(vran[i][0],vran[i][1],vran[i][3])
            ax1.set_yticks (ytick2,minor=True)
    plt.setp(ax1.get_yticklabels(),fontsize=10)
    plt.setp(ax1.get_yticklabels()[-1],visible=False)
    plt.setp(ax1.get_yticklabels()[0],visible=False)

    ax1.grid(axis='both')
    if isinstance(vran[i],list):
        if vran[i][0]==0: pass
        else: ax1.axhline(y=0,ls=':',c='k')
    else:   ax1.axhline(y=0,ls=':',c='k')

    if tevent!='none'     : ax1.axvline(x=tevent,ls='--',lw=2,c='k')
    if len(vline[i])!=0   :
       if   vline[i][-1] == 'v' :
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],ls=j[1],lw=j[2],c=j[3])
       elif vline[i][-1] == 'h' :
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],ls=j[1],lw=j[2],c=j[3]);print(j)
       elif vline[i][-1] == 'vp':
         for j in vline[i][:-1]  : ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4,linewidth=2.,linestyle='--')
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2]);ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'hp':
         for j in vline[i][:-1]  : ax1.axhspan(j[0],j[1],facecolor=j[2],alpha=0.4)
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],lw=2.,ls='--',c=j[2]);ax1.axhline(y=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'vph':
         for j in vline[i][:-1]  :
            ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4)
            ax1.axhline(y=j[3],ls=j[4],lw=j[5],c=j[6])
            ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2])
            ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])



    if i==0 :
      ax1.set_title(title,fontsize=12)

# plt.show()
# plt.tight_layout(h_pad=0.2)
  print("Save ",fps)
  plt.savefig(fps)
  plt.savefig(fps[:-4]+'.png')
  plt.close()

  return


def easy1Dplot_l(fps,tt,vv,ty,vline,ylab,vran,vstr,title,label,annotate,tevent='none',tkinfo=[],figsize=(8,11),anloc=(.1,.005),bottom=0.25,legloc=(0.9,0.2)):
  " time  info  : tt   = [ [t1,t2], [t2], [t3,t1,t4]                            \
    value info  : vv   = [ [v1,v2], [v2], [v3,v1,v4]                            \
    line style  : ty   = [ ['ro','b-'], ['r-'], ['r-','b-','g-]                 \
    string of vv: vstr = [ 'IMF By & Bz [nT]', 'Vsw [km/s]','CPCP [kV]' ]       \
    yrange of vv: vran = [ [-10,10], [300,500],[0,300] ]                        \
    title of fps: title= 'Jan 10 1997 summary plot'                             \
    eventtime for vertical bar :     tevent = 20000                             \
    tevent[0]   : must be the event time, makes thick black vertical lines      \
    tevent[1:]  : other timelines, makes thin red vertical lines                "

  print('easy1Dplot_l starts!')
  #print(annotate)

  if len(tkinfo)!=0 :
    tstr = mhdtime(tkinfo[0])
    dtmaj,dtmin,dtlab,xran = tkinfo[1:]
  else :
    info = runinfo(fps)
    tstr = mhdtime(info.ti)
    dtmaj,dtmin,dtlab,xran = tick_info(info.run)

  if xran[0]==-9999 and xran[-1]==9999 :
    xran = [ tt[-1][0][0], tt[-1][0][-1] ]

  print(dtmaj,dtmin,dtlab,xran)

  i       =  0
  xtick1  =  arange(xran[0],xran[1]+dtmaj,dtmaj)   # major tick location
  xtick2  =  arange(xran[0],xran[1]+dtmin,dtmin)   # minor tick location
  xlabel  =  mhdt2hrmi(xtick1,tstr,dtlab)          # create tick label

# fig=plt.figure(figsize=(8,11))
  if figsize==() :fig=plt.figure()
  else           :fig=plt.figure(figsize=figsize)

  for i in range(len(tt)):

    ax1=plt.subplot(len(tt),1,i+1)
    plt.subplots_adjust(hspace = .001)

    for j in range(len(tt[i])):
#      print(i,j)

      lst = ty[i][j].split(':')
#      breakpoint()
      if len(lst)==1: ax1.plot(tt[i][j],vv[i][j],ty[i][j])
      else:
#         print(lst[1],tt[i][j],vv[i][j])
         if lst[1]=='plot': ax1.plot(tt[i][j],vv[i][j],lst[0],label=label[j])
         if lst[1]=='step': ax1.step(tt[i][j],vv[i][j],lst[0],where='mid')
         if lst[1]=='yerr':
             ax1.errorbar(tt[i][j],vv[i][j][0],yerr=vv[i][j][1],fmt=lst[0],label=label[j])


#   ax1.text(0.01,0.9,vstr[i],ha='left',va='top', \
#            fontsize=10,transform=ax1.transAxes)

    if isinstance(vstr[i],list):
      vstr1,colr1,x1,y1 = vstr[i]
      for j in range(len(vstr1)):
         ax1.text(x1[j],y1[j],vstr1[j],ha='left',va='top',color=colr1[j],	\
             fontsize=10,transform=ax1.transAxes)
    else :
      ax1.text(0.99,0.9,vstr[i],ha='right',va='top', \
               fontsize=10,transform=ax1.transAxes)

#...xticks
    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)
    if i==len(tt)-1: ax1.set_xticklabels (xlabel,fontsize=10);ax1.set_xlabel('TIME [UT]')
    else           : ax1.set_xticklabels ([ '' for j in xlabel])
    ax1.set_xlim   (xran)
    ax1.tick_params(axis='x',which='major',direction='inout',length=6) #,width=2)
    ax1.tick_params(axis='x',which='minor',direction='inout',length=6)


#...yticks
    ax1.set_ylabel(ylab[i],fontsize=10)
    if isinstance(vran[i],list):
        ax1.set_ylim(vran[i][0],vran[i][1])
        if len(vran[i])>=3:
          ytick1 = arange(vran[i][0],vran[i][1]+vran[i][2],vran[i][2])
          ax1.set_yticks (ytick1)
          if len(vran[i])==4:
            ytick2 = arange(vran[i][0],vran[i][1],vran[i][3])
            ax1.set_yticks (ytick2,minor=True)
    plt.setp(ax1.get_yticklabels(),fontsize=10)
    plt.setp(ax1.get_yticklabels()[-1],visible=False)
    plt.setp(ax1.get_yticklabels()[0],visible=False)

    ax1.grid(axis='both')
    if isinstance(vran[i],list):
        if vran[i][0]==0: pass
        else: ax1.axhline(y=0,ls=':',c='k')
#    else:   ax1.axhline(y=0,ls=':',c='k')

    if tevent!='none'     : ax1.axvline(x=tevent,ls='--',lw=2,c='k')
    if len(vline[i])!=0   :
       if   vline[i][-1] == 'v' :
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],ls=j[1],lw=j[2],c=j[3])
       elif vline[i][-1] == 'h' :
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],ls=j[1],lw=j[2],c=j[3]);print(j)
       elif vline[i][-1] == 'vp':
         for j in vline[i][:-1]  : ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4,linewidth=2.,linestyle='--')
         for j in vline[i][:-1]  : ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2]);ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'hp':
         for j in vline[i][:-1]  : ax1.axhspan(j[0],j[1],facecolor=j[2],alpha=0.4)
         for j in vline[i][:-1]  : ax1.axhline(y=j[0],lw=2.,ls='--',c=j[2]);ax1.axhline(y=j[1],lw=2.,ls='--',c=j[2])
       elif vline[i][-1] == 'vph':
         for j in vline[i][:-1]  :
            ax1.axvspan(j[0],j[1],facecolor=j[2],alpha=0.4)
            ax1.axhline(y=j[3],ls=j[4],lw=j[5],c=j[6])
            ax1.axvline(x=j[0],lw=2.,ls='--',c=j[2])
            ax1.axvline(x=j[1],lw=2.,ls='--',c=j[2])


#    if len(tt[i])>=3: ax1.legend()

    if i==0 :
      ax1.set_title(title,fontsize=12)
    ax1.minorticks_on()
#    ax1.tick_params(axis='x', which='minor', bottom=False)
    #ax1.grid(axis='both')
  #plt.grid()
  handles, labels = ax1.get_legend_handles_labels()
  fig.legend(handles, labels, loc='upper right',bbox_to_anchor=legloc)

  plt.annotate(annotate,xy=anloc, xycoords='figure fraction')
  plt.subplots_adjust(bottom=bottom)
# plt.show()
# plt.tight_layout(h_pad=0.2)
  print("Save ",fps)
  plt.savefig(fps)
  plt.savefig(fps[:-4]+'.png')
  plt.close()

  return

def easylineplot(fps,xx,yy,lst,pst,xran,yran,xlab,ylab,tlab,vstr,twin=[],vline=[],hline=[],figsize=(8,11)):
  " draw line plots that shares the same x axis 				\
    xaxis info  : tt   = [ [t1,t2], [t2], [t3,t1,t4]                            \
    yaxis info  : vv   = [ [v1,v2], [v2], [v3,v1,v4]                            \
    line style  : lst  = [ ['ro','b-'], ['r-'], ['r-','b-','g-]                 \
    xrange      : xran = [ [minx,maxx,major tick space ,minor tick space], ..] 	\
    yrange      : yran = [ [minx,maxx,major tick space ,minor tick space], ..] 	\
    xy labels   : xlab,ylab = ['xgse','xgse',...]				\
    plot text   : vstr = [ 'IMF By & Bz [nT]', 'Vsw [km/s]','CPCP [kV]' ]       \
    plot title  : tlab - [ 'velocity','temerature',...]				\
    ===================								\
    twin        : set values to share x axis and plot yvalues 			\
                  twin = [yy,lst,pst,yran,ylab]					\
    vline/hline : set vertical/horizontal lines					\
                  vline =[ [loc,col],[loc,col], ]				"

  print('easylineplot starts!')

  if len(twin)!=0  : xtw,ytw,lstw,pstw,yrtw,yltw = twin

  fig=plt.figure(figsize=figsize)

  for i in range(len(xx)):
    ax1=plt.subplot(len(xx),1,i+1)
    plt.subplots_adjust(hspace = .003)

    for j in range(len(xx[i])):
      if   pst[i][j]=='step'    : ax1.step(xx[i][j],yy[i][j],lst[i][j])
      elif pst[i][j]=='semilogy': ax1.semilogy(xx[i][j],yy[i][j],lst[i][j])
      else                      : ax1.plot(xx[i][j],yy[i][j],lst[i][j])

    ax1.set_title(tlab[i])
#   ax1.text(0.01,0.9,vstr[i],ha='left',va='top',transform=ax1.transAxes)
    ax1.text(0.99,0.9,vstr[i],ha='right',va='top',transform=ax1.transAxes)
#...xticks
    xtick1  =  arange(xran[i][0],xran[i][1]+xran[i][2],xran[i][2])   # major tick location
    ax1.set_xticks (xtick1)
    if len(xran)==4:
      xtick2  =  arange(xran[i][0],xran[i][1]+xran[i][3],xran[i][3]) # minor tick location
      ax1.set_xticks (xtick2,minor=True)
    ax1.set_xlabel (xlab[i])
    if i!=len(xx)-1: ax1.set_xticklabels ([ '' for j in xtick1])
    ax1.set_xlim   (xran[i][0],xran[i][1])
    plt.setp(ax1.get_xticklabels(),fontsize=8)
    ax1.tick_params(axis='x',which='major',direction='inout',length=6) #,width=2)
    ax1.tick_params(axis='x',which='minor',direction='inout',length=6)

#...yticks
    ytick1  =  arange(yran[i][0],yran[i][1]+yran[i][2],yran[i][2])   # major tick location
    ax1.set_yticks (ytick1)
    if len(yran)==4:
      ytick2  =  arange(yran[i][0],yran[i][1]+yran[i][3],yran[i][3]) # minor tick location
      ax1.set_yticks (ytick2,minor=True)
    if len(twin)!=0 : ax1.set_ylabel (ylab[i],color=lst[i][0][0])
    else            : ax1.set_ylabel (ylab[i])
    ax1.set_ylim   (yran[i][0],yran[i][1])
    plt.setp(ax1.get_yticklabels(),fontsize=8)
    plt.setp(ax1.get_yticklabels()[-1],visible=False)
    plt.setp(ax1.get_yticklabels()[0],visible=False)
#   ax1.grid(True)
    ax1.grid(axis='x')

#...vlines
    if len(vline)!=0:
      if len(vline[i])!=0:
        loc,col=vline[i]
        for k in range(len(loc)): ax1.axvline(x=loc[k],ls='-',lw=2,c=col[k])

#...hlines
    if len(hline)!=0:
      if len(hline[i])!=0:
        loc,col=hline[i]
        for k in range(len(loc)): ax1.axhline(y=loc[k],ls='-',lw=2,c=col[k])
    if yran[i][0] <0 and yran[i][1] >0: ax1.axhline(y=0,ls=':',c='k')
#   ax1.axhline(y=0.,ls=':',lw=2,c=k)

#### share x-axis and plot yvalues
    if len(twin)!=0 :
      ax2 = ax1.twinx()
      for j in range(len(xtw[i])):
        if pstw[i][j]=='step': ax2.step(xtw[i][j],ytw[i][j],lstw[i][j])
        else                 : ax2.plot(xtw[i][j],ytw[i][j],lstw[i][j])

#... twin yticks
      ytk1  =  arange(yrtw[i][0],yrtw[i][1]+yrtw[i][2],yrtw[i][2])   # major tick location
      ytk2  =  arange(yrtw[i][0],yrtw[i][1]+yrtw[i][3],yrtw[i][3])   # minor tick location
      ax2.set_ylabel (yltw[i],color=lstw[i][j][0])
      ax2.set_yticks (ytk1)
      ax2.set_yticks (ytk2,minor=True)
      ax2.set_ylim   (yrtw[i][0],yrtw[i][1])
      plt.setp(ax2.get_yticklabels(),fontsize=8)
      plt.setp(ax2.get_yticklabels()[-1],visible=False)
      plt.setp(ax2.get_yticklabels()[0],visible=False)
      ax2.set_xlim   (xran[i][0],xran[i][1])


# plt.show()
# plt.tight_layout(h_pad=0.2)
  print("Save ",fps)
  plt.savefig(fps)

  plt.close()

  return


def easy2Dplot(nrow,ncol,fps,xran,yran,xx,yy,val,vstr,title,ftitle):
  "plot 2D"

####### draw graphs

  fig = plt.figure(figsize=(11,8))

  for i in range(len(val)):

    ax1      = fig.add_subplot(nrow,ncol,i+1,aspect='equal')
    ax1.tick_params(axis='both',which='major',labelsize=7)

    x,y      = np.meshgrid(xx,yy)
    pkg_plot = mhd_vpkg(vstr[i])


    if pkg_plot[5]==1:
      cs    = ax1.contourf(x,y,val[i],levels=pkg_plot[3], \
              cmap=pkg_plot[1],norm=LogNorm())
    else :
      cs    = ax1.contourf(x,y,val[i],levels=pkg_plot[3], \
              cmap=pkg_plot[1],extend='both')

#   cax1  = make_axes_locatable(ax1)
#   cax2  = cax1.append_axes("right",size="3%",pad="5%")
#   cb    = fig.colorbar(cs,cax=cax2,ticks=pkg_plot[4])
#   cl    = plt.getp(cb.ax, 'ymajorticklabels')
#   plt.setp(cl, fontsize=7)

####### tick infomation

    ax1.yaxis.set_major_locator(MultipleLocator(yran[2]))
    ax1.yaxis.set_minor_locator(MultipleLocator(yran[3]))
    ax1.xaxis.set_major_locator(MultipleLocator(xran[2]))
    ax1.xaxis.set_minor_locator(MultipleLocator(xran[3]))
    ax1.set_xlim(xran[0],xran[1])
    ax1.set_ylim(yran[0],yran[1])
    ax1.grid(True)

##### titles for the figures
    vmax,vmin = maxmin_ran([xran[0],xran[1]],[yran[0],yran[1]],xx,yy,val[i])
    lmax = "Max: %.2f "%vmax
    lmin = "Min: %.2f "%vmin

    ax1.text(0.0,1.0,title[i]   ,fontsize=9,ha='left' ,va='bottom' ,transform=ax1.transAxes)
    ax1.text(1.0,1.0,pkg_plot[0],fontsize=9,ha='right',va='bottom' ,transform=ax1.transAxes)
    ax1.text(0.0,0.0,lmin       ,fontsize=8,ha='left' ,va='bottom',transform=ax1.transAxes)
    ax1.text(1.0,0.0,lmax       ,fontsize=8,ha='right',va='bottom',transform=ax1.transAxes)

  plt.figtext(0.5,0.95,ftitle,fontsize=10,ha='center')
  plt.savefig(fps,orientation='landscape')
  plt.close()

  return



def magic_summ(nrow,ncol,fname,fout):
  " make a sumary plot of ps files"

  k     = -1
  name3 =' '
  for i in range(nrow) :
    name1 = ' '
    name2 = 'test%02d.'%(i+1)+fout.split('.')[-1]
    for j in range(ncol):
       k    = k+1
       name1= name1+fname[k]+' '
    cmd1='convert -trim'+name1+' +append '+name2
    print(cmd1)
    os.system(cmd1)
    name3=name3+name2+' '

  cmd2='convert '+name3+' -append '+fout
  print(cmd2)
  os.system(cmd2)

  cmd3='rm '+name3
  print(cmd3)
  os.system(cmd3)

  return



#!/bin/bash
# Written by Jaewoon Jung, on 03/07/2024.
# Some of the code used are based on the code written by Hyunju K. Connor.

#Run example:
#python Msh_xray.py name 0 path jel 30 75 105 1 180 210 1 0 0 1
#mshpy_xray('name',0,'path','jel',30,75,105,1,180,210,1,0,0,1)
#will make simulated X-ray emission measurement data observed from 30 Re away from the Earth in 'path'/xray.'name'/data
#and image in 'path'/xray.'name'/ps
#Solar wind condition should be pre-exist as 'SW_cond.txt' as in current directory. 
#Format of the file should be following:
#Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
#e.g. 
#1967 1 0 00 2 -2 -5 -400 0 0 10 2 10 6


import sys
import os
from Mshpy.xraysc_sight_Msh import main as Mshpy_xray_sim
from Mshpy.plot_rates_Msh import main as Mshpy_xray_plot

def mshpy_xray(RUN, TI, Dout, model_bs, dsc, the1, the2, dthe, phi1, phi2, dphi, XRAYPS, MSHPY, ALL):
    the1_ = the1 - 90
    the2_ = the2 - 90

    ran = f"xray,xray:{phi1}:{phi2}:10:2:{the1_}:{the2_}:10:2"

    if MSHPY == 1:
        PYE = Mshpy_xray_sim
        Din = "./modeling"
        Ddata = os.path.join(Dout, f"xray.{RUN}/data")

        if not os.path.exists(Dout):
            os.mkdir(Dout)

        LL = str(TI).zfill(6)
        OP1 = f"-xsc {dsc} -ysc 0 -zsc 0"
        OP2 = "-xlk 0 -ylk -1 -zlk 0"
        OP3 = f"-the1 {the1} -the2 {the2} -dthe {dthe} -phi1 {phi1} -phi2 {phi2} -dphi {dphi}"
        fout = os.path.join(Ddata, f"{RUN}.{LL}")
        cmd = f"python PYE -path {Din} {OP1} {OP2} {OP3} -fo {fout} -model {model_bs}"
        print(arg)
        arg=cmd.split()[1:]
        PYE(arg)

    if XRAYPS == 1:
        PYE = Mshpy_xray_plot
        Ddata = os.path.join(Dout, f"xray.{RUN}/data")
        Dps = os.path.join(Dout, f"xray.{RUN}/ps")

        if not os.path.exists(Dps):
            os.mkdir(Dps)

        LL = str(TI).zfill(6)
        TT2 = TI + 1
        LL2 = str(TT2).zfill(6)
        fin = f"{Ddata}/{RUN}.sc.{LL}:{Ddata}/{RUN}.sc.{LL2}"
        fbl = "none"
        fmp = "none"
        tash = 0
        cmd = f"python PYE {Dps} {fin} {fbl} {fmp} {ran} {tash}"
        print(arg)
        arg=cmd.split()
        PYE(arg)

    if ALL == 1:
        PYE = Mshpy_xray_sim
        Din = "./modeling"
        Ddata = os.path.join(Dout, f"xray.{RUN}/data")

        if not os.path.exists(Dout):
            os.mkdir(Dout)

        if not os.path.exists(os.path.join(Dout, f"xray.{RUN}")):
            os.mkdir(os.path.join(Dout, f"xray.{RUN}"))

        if not os.path.exists(Ddata):
            os.mkdir(Ddata)

        LL = str(TI).zfill(6)
        TT2 = TI + 1
        LL2 = str(TT2).zfill(6)

        xsc = 0
        ysc = dsc
        zsc = 0

        xlk = 0
        ylk = -1
        zlk = 0

        ysc2 = 0
        zsc2 = dsc

        ylk2 = 0
        zlk2 = -1

        OP1 = f"-xsc {xsc} -ysc {ysc} -zsc {zsc}"
        OP1_2 = f"-xsc {xsc} -ysc {ysc2} -zsc {zsc2}"

        OP2 = f"-xlk {xlk} -ylk {ylk} -zlk {zlk}"
        OP2_2 = f"-xlk {xlk} -ylk {ylk2} -zlk {zlk2}"

        the1_2 = phi1 - 90
        the2_2 = phi2 - 90
        phi1_2 = the1 + 90
        phi2_2 = the2 + 90

        OP3 = f"-the1 {the1} -the2 {the2} -dthe {dthe} -phi1 {phi1} -phi2 {phi2} -dphi {dphi}"
        OP3_2 = f"-the1 {the1_2} -the2 {the2_2} -dthe {dthe} -phi1 {phi1_2} -phi2 {phi2_2} -dphi {dphi}"

        fout = os.path.join(Ddata, f"{RUN}.sc.{LL}")
        fout_2 = os.path.join(Ddata, f"{RUN}.sc.{LL2}")

        cmd = f"python PYE -path {Din} {OP1} {OP2} {OP3} -fo {fout} -model {model_bs}"
        arg=cmd.split()[1:]
        print(arg)
        PYE(arg)

        cmd = f"python PYE -path {Din} {OP1_2} {OP2_2} {OP3_2} -fo {fout_2} -model {model_bs}"
        arg=cmd.split()[1:]
        print(arg)
        PYE(arg)

        PYE = Mshpy_xray_plot
        Ddata = os.path.join(Dout, f"xray.{RUN}/data")
        Dps = os.path.join(Dout, f"xray.{RUN}/ps")

        if not os.path.exists(Dps):
            os.mkdir(Dps)

        fin = f"{Ddata}/{RUN}.sc.{LL}:{Ddata}/{RUN}.sc.{LL2}"
        fbl = "none"
        fmp = "none"        
        tash = 0
        cmd = f"python PYE {Dps} {fin} {fbl} {fmp} {ran} {tash}"
        arg=cmd.split()[2:]
        print(arg)
        PYE(arg)

if __name__ == "__main__":
    RUN = sys.argv[1]
    TI = int(sys.argv[2])
    Dout = sys.argv[3]
    model_bs = sys.argv[4]
    dsc = float(sys.argv[5])
    the1 = float(sys.argv[6])
    the2 = float(sys.argv[7])
    dthe = float(sys.argv[8])
    phi1 = float(sys.argv[9])
    phi2 = float(sys.argv[10])
    dphi = float(sys.argv[11])
    MSHPY = int(sys.argv[12])
    XRAYPS = int(sys.argv[13])
    ALL = int(sys.argv[14])

    mshpy_xray(RUN, TI, Dout, model_bs, dsc, the1, the2, dthe, phi1, phi2, dphi, XRAYPS, MSHPY, ALL)




import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp2d
from astropy.io import fits

cd=1/156
fits_image_filename_n='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa_density.fits'
hdu_n=fits.open(fits_image_filename_n)
data_n=hdu_n[0].data
fits_image_filename_v='/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa_velocity.fits'
hdu_v=fits.open(fits_image_filename_v)
data_v=hdu_v[0].data
data_T=(data_v!=0)*(1+64/3*(1-data_v**2))
x=np.array([1-(i-133)*cd for i in range(512)])
y=np.array([(i-10)*cd for i in range(491)])
print(np.min(x),np.max(x))
print(np.min(y),np.max(y))
n=interp2d(x,y,data_n)
v=interp2d(x,y,data_v)

def BS():
    a=open('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/BS.txt',"w")
    n_r=0
    for phi in np.arange(0,90.5,0.5):
        r=0
        r_mp=0
        while (n_r==0) & (r<=3.6):
            n_r=n(r*np.cos(phi*np.pi/180),r*np.sin(phi*np.pi/180))
            r+=.01
        r_mp=r
        if n_r>0:
            while n_r>0:
                n_r=n(r*np.cos(phi*np.pi/180),r*np.sin(phi*np.pi/180))
                r+=.01
            a.write('%f %f %f\n'%(phi,r_mp,r))
    a.close()
    return

#BS()

fig, ax = plt.subplots(1,3,figsize=(12,4))
plot_n=ax[0].pcolormesh(x, y, data_n)
plot_v=ax[1].pcolormesh(x, y, data_v)
plot_T=ax[2].pcolormesh(x, y, data_T)
ax[0].grid()
ax[1].grid()
ax[2].grid()
ax[0].set_xlabel('x/R_MP')
ax[0].set_ylabel('y(z)/R_MP')
ax[1].set_xlabel('x/R_MP')
#ax[1].set_ylabel('y(z)/R_MP')
ax[2].set_xlabel('x/R_MP')
#ax[2].set_ylabel('y(z)/R_MP')
fig.suptitle('Spreiter ($\gamma=5/3$, $M_\infty=8$)')
c1=fig.colorbar(plot_n,ax=ax[0])
c1.ax.set_title('n/n_SW')
c2=fig.colorbar(plot_n,ax=ax[1])
c2.ax.set_title('v/v_SW')
c3=fig.colorbar(plot_T,ax=ax[2])
c3.ax.set_title('T/T_SW')
plt.savefig('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa.png')


b=pd.read_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/BS.txt',header=None,sep='\s+')
b=b[b[0]%7.5==0]
b.columns=['phi','rmp','rbs']
#b_2=b.copy()
#b_2.phi=-b_2.phi
#b=pd.concat([b,b_2])

phi,f=np.mgrid[-82.5:90:7.5,0.1:1:.1]
phif=np.array([phi.flatten(),f.flatten()]).T
c=pd.DataFrame(phif)
c.columns=['phi','f']
d=pd.merge(c,b,on='phi')
d['r']=d.rmp+d.f*(d.rbs-d.rmp)
d['x']=d.r*np.cos(d.phi*np.pi/180)
d['y']=d.r*np.sin(d.phi*np.pi/180)
d['ni']=d.apply(lambda x:n(x.x,x.y)[0],axis=1)
d['V']=d.apply(lambda x:v(x.x,x.y)[0],axis=1)
d['Ti']=1+64/3*(1-d.V**2)
d.to_csv('/Volumes/easystore/openggcm_run/Msh_Nstep/Spreiter/ssa.txt',float_format='%.4f',index=False)



#!/usr/bin/python
#
# get sc orbit file to obtain MHD values along spacecraft orbit


import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")
sys.path.append(HOME+"/svn/hkc-project/trunk/src/swdata/")

import locale
from jrmod01            import *
from numpy              import arange
from openggcm_info      import mhdtime,mhdt2hrmi
from swdata		import interpolate_data
import os

def ldaf(filename):
    with open(filename) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith('dd'):
                lda=num
#                print ('found at line:', num)
    return lda

def read_CLFGM(fi,vtype='string'):
  "read Cluster FGM data downloaded from SPDweb		\
   vtype: string or real to return string/real arrays   "

  Err = '-1.00000E+31'
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  bx=[];by=[];bz=[]
  xl=[];yl=[];zl=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and w[4]!=Err and \
       w[5]!=Err and w[6]!=Err and w[7]!=Err                   :

      date.append(w[0])
      time.append(w[1])
      bx.append(w[2])
      by.append(w[3])
      bz.append(w[4])
      x1=locale.format('%f',locale.atof(w[5])/re)
      y1=locale.format('%f',locale.atof(w[6])/re)
      z1=locale.format('%f',locale.atof(w[7])/re)
      xl.append(x1)
      yl.append(y1)
      zl.append(z1)

  if vtype=='real':
    bx = [locale.atof(i) for i in bx ]
    by = [locale.atof(i) for i in by ]
    bz = [locale.atof(i) for i in bz ]
    xl = [locale.atof(i) for i in xl  ]
    yl = [locale.atof(i) for i in yl  ]
    zl = [locale.atof(i) for i in zl  ]

  return date,time,bx,by,bz,xl,yl,zl


def read_CLCIS(fi,vtype='string'):
  "read Cluster CIS data			\
   vtype : string to return string array 	\
           real   to return real array		"

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)
#  print(lda)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nden=[];Tpar=[];Tper=[];vx=[];vy=[];vz=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err  and w[4]!=Err  and  \
       w[5]!=Err and w[9]!=Err and w[10]!=Err and w[11]!=Err      :

      qual= locale.atoi(w[5])
      if qual ==1 or qual==2: # grep good data (1: use with caution, 2: ok)
        date.append(w[0])
        time.append(w[1])
        nden.append(w[2])
        Tpar.append(w[3])
        Tper.append(w[4])
        vx.append(w[9])
        vy.append(w[10])
        vz.append(w[11])

  if vtype=='real':
    nden = [locale.atof(i) for i in nden ]
    Tpar = [locale.atof(i) for i in Tpar ]
    Tper = [locale.atof(i) for i in Tper ]
    vx   = [locale.atof(i) for i in vx   ]
    vy   = [locale.atof(i) for i in vy   ]
    vz   = [locale.atof(i) for i in vz   ]

  return date,time,nden,Tpar,Tper,vx,vy,vz



def create_CLdata(sc,fi1,fi2,lda1,lda2,t1,t2,t3,t4,av):
  "interpolate Cluster FGM and CIS data"

## interpolate ACE data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','xgse','ygse','zgse', \
          'rr','tpar','tper','vxgse','vygse','vzgse'        ]
  fo   = [ sc+'.'+i for i in name ]

  date1,time1,bx,by,bz,xl,yl,zl          = read_CLFGM(fi1,lda1)
  date2,time2,np,tpar,tper,vx,vy,vz      = read_CLCIS(fi2,lda2)

  date = [date1,date1,date1,date1,date1,date1,date2,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time1,time1,time1,time2,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,xl,yl,zl,np,tpar,tper,vx,vy,vz]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create orbit, the OpenGGCU input starting from t3
  name = ['xgse','ygse','zgse' ]
  fo   = [ sc+'.'+i for i in name ]
  create_CLorbit(fo,sc+'.GSEorbit',t1,t2,t3,t4)
  return


def create_CLorbit(fi,fo,t1,t2,t3,t4):
  " create swdata, the OpenGGCM input, in the following format                  \
    T(min) bx(nT) by(nT) bz(nT) vx(km/s) vy(km/s) vz(km/s) N(cc) P(pP)          \
    ----------------------------------------------------------------------------\
    fi   : input data lists in the folloing order                               \
           [ bx,by,bz,vx,vy,vz,nn,pp ]                                          \
    fo   : output data                                                          \
    t1   : start time of solar wind data                                        \
    t2   : end   time of solar wind data                                        \
    t3   : start time of OpenGGCM run                                           \
    t4   : end   time of OpenGGCM run                                           \
    ----------------------------------------------------------------------------"

  vv  = []
  tt1 = epo1c(t1)
  tt2 = epo1c(t2)
  tt3 = epo1c(t3)
  tt4 = epo1c(t4)
  fmt = "%10.2f %8.4f %8.4f %8.4f\n"

  for i in range(len(fi)):
    x=read_tseries(fi[i],tt1,tt2)
    if i==0: vv.append(x[6])
    vv.append(x[7])

  f=open(fo,'w')
  for j in range(len(vv[0])):
   if vv[0][j] >=tt3 and vv[0][j]<=tt4:
     t=(vv[0][j]-tt3)
     ll = fmt%(t,vv[1][j],vv[2][j],vv[3][j])
     f.write(ll)
  f.close()
  return


def main(argv):

  sc=argv[0]                    # satellite info
  f1=argv[1]                    #MFI data: date time bxgse bygse bzgse
  f2=argv[2]                    #SWE data: date time vth rr xgse ygse zgse vxgse vygse vzgse
  t1=argv[3]                    #starttime for interpolation
  t2=argv[4]                    #endtime   for interpolation
  t3=argv[5]                    #starttime for MHD simulation
  t4=argv[6]                    #endtime   for MHD simulation
  av=argv[7]                    #time interval of interpolation in sec

  lda=argv[8].split(':')        #the line where data starts
  lda1=locale.atoi(lda[0])-1
  lda2=locale.atoi(lda[1])-1

  print(lda,lda1,lda2)

  create_CLdata(sc,f1,f2,lda1,lda2,t1,t2,t3,t4,av)


  return


if __name__=='__main__':
   main(sys.argv[1:])



#!/usr/bin/python
# openggcm modules

import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")

import locale
import matplotlib.pyplot as     plt
from   run_info          import openggcm_run
from   datetime         import datetime,timedelta

class mhdtime(object):
  "create class of mhdtime for easy access"

  def __init__(self,t_str) :
      self.tstr=t_str
      w=self.tstr.split(':')

      self.yy=locale.atoi(w[0])
      self.mm=locale.atoi(w[1])
      self.dd=locale.atoi(w[2])
      self.hr=locale.atoi(w[3])
      self.mi=locale.atoi(w[4])
      self.se=locale.atof(w[5])


      Mon  = ['Jan','Feb','Mar','Apr','May','Jun', \
              'Jul','Aug','Sep','Oct','Nov','Dec']

      self.date  = '%02d-'%self.dd+Mon[self.mm-1]+'-%04d'%self.yy
      self.time  = '%02d:%02d UT'%(self.hr,self.mi)
      self.name  = self.date+' '+self.time
      self.mdate = '%04d'%self.yy+Mon[self.mm-1]+'%02d'%self.dd
      return


class runinfo(object):
  "read run name, data type (iof,ioc,3df,etc.), and mhdtime from filename   	\
   ex. name='iof.hjk2001/hjk2001.iof.003600'				    	\
       name.dir = 'iof.hjk2001' 					    	\
       name.run = 'hjk2001'						    	\
       name.type= 'iof'								\
       name.mhdt= '003600'							\
       self.ti  = '1967:01:01:00:00:0.0' :start time of mhdrun			"

  def __init__(self,fname):
   self.fname=fname

   s1        = fname.split('/')
   s2        = s1[-1].split('.')

   ll=' '
   for i in range(len(s1)-1) : ll=ll+s1[i]+'/'

   self.dir  = ll
   self.name = s1[-1]
   self.run  = s2[0]
   self.type = s2[1]
   self.mhdt = s2[2]

   times     = openggcm_run(self.run)
   #print(times)
   self.ti   = times[0]
   self.tf   = times[1]
   self.dipt = times[2]
   tstr      = [ int(locale.atof(i)) for i in self.ti.split(':') ]
   tmhd0     = datetime(tstr[0],tstr[1],tstr[2],tstr[3],tstr[4],tstr[5] )
   tstr1     = [ int(locale.atof(i)) for i in self.tf.split(':') ]
   tmhd1     = datetime(tstr1[0],tstr1[1],tstr1[2],tstr1[3],tstr1[4],tstr1[5] )
   self.dur  = (tmhd1-tmhd0).total_seconds()
#   print(self.dur)

   tstrs     = mhdtime(self.ti)
   self.date = tstrs.date
   self.time = tstrs.time
   self.ddtt = tstrs.name
   self.mdate= tstrs.mdate

   return



def mhdt2tstr(mhdt,starttime):
  "convert mhdt to time string.                                                 \
   ex: convert 010800 to 1967:01:01:03:00:0.0 if startime=1967:01:01:00:00:0.0  "

  time=mhdtime(starttime)

  ay = int(mhdt/364)

  a1 = int(mhdt/3600)
  a2 = int(mhdt%3600)/60

  yy = time.yy
  mm = time.mm
  dd = time.dd
  hr = time.hr+a1
  mi = time.mi+a2
  se = time.se+mhdt%60.


  if yy%4 == 0    : Mon = [31,29,31,30,31,30,31,31,30,31,30,31]
  else            : Mon = [31,28,31,30,31,30,31,31,30,31,30,31]

  if se>=60       : se=se-60 ; mi=mi+1
  if mi>=60       : mi=mi-60 ; hr=hr+1
  if hr>=24       : d1=int(hr/24);hr=hr-24*d1 ; dd=dd+d1
  if dd>Mon[mm-1] : dd=1     ; mm=mm+1
  if mm>12        : mm=1     ; yy=yy+1


  tstr="%4d:%02d:%02d:%02d:%02d:%05.2f"%(yy,mm,dd,hr,mi,se)

  return tstr


def mhdt2tstr(mhdt,starttime):
  "convert mhdt to time string.							\
   ex: convert 010800 to 1967:01:01:03:00:0.0 if startime=1967:01:01:00:00:0.0	"

  time=mhdtime(starttime)

  a1 = int(mhdt/3600)
  a2 = int(mhdt%3600)/60

  yy = time.yy
  mm = time.mm
  dd = time.dd
  hr = time.hr+a1
  mi = time.mi+a2
  se = time.se+mhdt%60.


  if yy%4 == 0    : Mon = [31,29,31,30,31,30,31,31,30,31,30,31]
  else            : Mon = [31,28,31,30,31,30,31,31,30,31,30,31]

  if se>=60       : se=se-60 ; mi=mi+1
  if mi>=60       : mi=mi-60 ; hr=hr+1
  if hr>=24       : d1=int(hr/24);hr=hr-24*d1 ; dd=dd+d1
  if dd>Mon[mm-1] : dd=1     ; mm=mm+1
  if mm>12	  : mm=1     ; yy=yy+1


  tstr="%4d:%02d:%02d:%02d:%02d:%05.2f"%(yy,mm,dd,hr,mi,se)

  return tstr


def tstr2mhdt(tstr,starttime):
  "convert tstr to mhdt time							\
   ex: convert 1999:01:01:01:00:0.0 to 3600 if starttime=1999:01:01:00:00:0.0	"

  tt1=epo1c(starttime)
  tt2=epo1c(tstr)
  mhdt=tt2-tt1
  return mhdt


def grep_swdata(fsw,tsh,info):
  "read solar wind data for your taste"
  ftest=fsw.split('/')
  if ftest[-1]=='in.'+info.run:
    tsw,sw     = read_swdata(fsw,tsh)
    swtype     = 'input'
    vstr_add   = ' : OpenGGCM Input'
  elif ftest[-1]=='swdata.omni':
    tsw,sw     = read_swomni(fsw,0.)
    swtype     = 'OMNI'
    vstr_add   = ' : OMNI data'
  else :
    tsw,sw     = read_timehist(fsw,'SW-01',120)
    vstr_add   = ' at (16,16,16) Re'
    swtype     = 'SW01'
  return tsw,sw,swtype,vstr_add


def read_swomni(fi,tsh):
  "read swdata"

  pm=1.672614e-27               # proton mass
  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  tt=[];rr=[];pp=[]
  bx=[];by=[];bz=[]
  vx=[];vy=[];vz=[]
  pd=[];symh=[]

  for s in l:
    w=s.split(None)
    if len(w)>2:

      tt.append(locale.atof(w[0])*60.+tsh)
      bx.append(locale.atof(w[1]))
      by.append(locale.atof(w[2]))
      bz.append(locale.atof(w[3]))
      vx.append(locale.atof(w[4]))
      vy.append(locale.atof(w[5]))
      vz.append(locale.atof(w[6]))
      rr.append(locale.atof(w[7]))
      pp.append(locale.atof(w[8]))
      vx1=locale.atof(w[4])
      vy1=locale.atof(w[5])
      vz1=locale.atof(w[6])
      rr1=locale.atof(w[7])
      pd1=(vx1**2+vy1**2+vz1**2)*rr1*pm*1.e21    # dynamic pressure in nPa: rho v**2
      pd.append(pd1)
      symh.append(locale.atof(w[9]))

  vv=[bx,by,bz,vx,vy,vz,rr,pp,pd,symh]
  return tt,vv



def read_swdata(fi,tpush=0):
  "read swdata from in.$RUN 						\
   tpush : shifted time in seconds to account for solar wind propagation "

  test0=fi.split('/')
  test1=test0[-1].split('.')
  if test1[0]=='in' : wlen=13
  else              : wlen=12

  pm=1.672614e-27 		# proton mass
  f = open(fi,'r')
  l = f.read().split('\n')
  f.close()

  i=0
  tt=[];rr=[];pp=[];pd=[]
  bx=[];by=[];bz=[];bt=[]
  vx=[];vy=[];vz=[];vt=[];fl=[]

  for s in l :

   w=s.split(None)

   if len(w)==wlen:
      tt1=locale.atof(w[0])*60.+tpush
      bx1=locale.atof(w[1])
      by1=locale.atof(w[2])
      bz1=locale.atof(w[3])
      vx1=locale.atof(w[4])
      vy1=locale.atof(w[5])
      vz1=locale.atof(w[6])
      rr1=locale.atof(w[7])
      pp1=locale.atof(w[8])
      bt1=(bx1**2+by1**2+bz1**2)**0.5
      vt1=(vx1**2+vy1**2+vz1**2)**0.5
#     pd1=1./2.*pm*rr1*(vx1**2+vy1**2+vz1**2)*1.e21  	# dynamic pressure in nPa: rho v**2/2
#     pd1=pm*rr1*(vx1**2+vy1**2+vz1**2)*1.e21  		# dynamic pressure in nPa: rho v**2
      pd1=(vx1**2+vy1**2+vz1**2)*rr1*pm*1.e21		# dynamic pressure in nPa: rho v**2
      fl1=vt1*rr1*1e5
      tt.append(tt1)
      bx.append(bx1)
      by.append(by1)
      bz.append(bz1)
      bt.append(bt1)
      vx.append(vx1)
      vy.append(vy1)
      vz.append(vz1)
      vt.append(vt1)
      rr.append(rr1)
      pp.append(pp1)
      pd.append(pd1)
      fl.append(fl1)

  swdata=[bx,by,bz,vx,vy,vz,rr,pp,pd,bt,vt,fl]
  return tt,swdata


def mhdt2hrmi(ticks,time,tint):
  "create labels for mhdtimes in \"hr:mi\" format.                              \n \
   tcks : tick locations                                                       \n \
   time  : class mhdtime                                                        \n \
   tint  : interval between tick labels                                         \n "

  labels=[]

  for x in ticks :

    if (x-ticks[0])%tint==0 :
      a1 = int(x/3600)
      a2 = int(x%3600)/60

      hr = time.hr+a1
      mi = time.mi+a2

      while (mi >= 60): hr = hr + 1 ; mi = mi-60
      while (hr >= 24): hr = hr -24

      fmt ='%02d:%02d'%(hr,mi)

    else         :
      fmt = ' '

    labels.append(fmt)

  return labels


def epo1c(c):
  w=c.split(':')
  return(epo1(locale.atoi(w[0]),
              locale.atoi(w[1]),
              locale.atoi(w[2]),
              locale.atoi(w[3]),
              locale.atoi(w[4]),
              locale.atof(w[5])))

def epo1(iy,mo,id,ih,mi,se):
  if iy < 200 : iy = 1900 + iy
  i1=367*iy
  i2=int((mo+9)/12)
  i2=(i2+iy)*7
  i2=int(i2/4)
  i3=int((275*mo)/9)
  i4=100*iy+mo
  d1=(1.0)
  if (i4-190002.5) < 0.0 : d1=(-1.0)
  rr = ( i1 - i2 + i3 + id + 1721013.5 -0.5*d1 +0.5 )*86400.0
  rr = rr + 3600.0*ih + 60.0*mi +se
  return(rr)


def fdoy2time(yr,fr):
  "convert fraction of DOY (day of year) to yyyy:mm:dd:hr:mi:se    \
   (yr,fr) = (2005,134.00)  --->  2005:05:15:00:00:0.0) "

  dy    = int(fr)
  mm,dd = doy2date(yr,dy)
  fr    = (fr-dy)*86400.
  hr    = int(fr/3600.)
  mi    = int((fr-hr*3600.)/60.)
  se    = fr-hr*3600.-mi*60.
  ctime = "%04d:%02d:%02d:%02d:%02d:%05.2f"%(yr,mm,dd,hr,mi,se)
  return ctime


def time2fdoy(ctime):
  "convert yyyy:mm:dd:hr:mi:se to fraction of day"

  tt   = ctime.split(':')
  yy   = locale.atoi(tt[0])
  mm   = locale.atoi(tt[1])
  dd   = locale.atoi(tt[2])
  hr   = locale.atoi(tt[3])
  mi   = locale.atoi(tt[4])
  se   = locale.atof(tt[5])

  doy  = date2doy(yy,mm,dd)
  fr   = doy+hr/24.+mi/24./60.+se/24./60./60.
  return yy,fr


def doy2date(yy,dy):
  "convert day of year (DOY) to date"

  if yy%4==0 : day=[31,29,31,30,31,30,31,31,30,31,30,31]
  else       : day=[31,28,31,30,31,30,31,31,30,31,30,31]

  for i in range(len(day)):
    if dy <= day[i] : break
    else            : dy=dy-day[i]
  mm=i+1
  return mm,dy


def date2doy(yy,mm,dd):
  "convert date to day of year"

  if yy%4==0 : day=[31,29,31,30,31,30,31,31,30,31,30,31]
  else       : day=[31,28,31,30,31,30,31,31,30,31,30,31]

  doy=0
  for i in range(mm-1): doy=doy+day[i]
  doy=doy+dd

  return doy


def main():
  "test functions"

  tstr = "1998:01:10:23:48:33.0"
  time = mhdtime(tstr)

  print ("test class mhdtime")
  print (time.yy,time.mm,time.dd,time.hr,time.mi,time.se)
  print (time.date,time.time,time.name)


  name = "iof.djl00703a/djl00703a.iof.013200"
  info = runinfo(name)
  print (info.run,info.type,info.mhdt,info.dir,info.name)


  starttime = "1995:02:28:00:00:25.0"
  timestr   = mhdt2tstr(60*60*24+3663,starttime)
  print (timestr)

  starttime = "1996:02:28:00:00:0.0"
  timestr   = mhdt2tstr(60*60*24+3660,starttime)
  print (timestr)

  starttime = "1996:12:31:00:00:0.0"
  timestr   = mhdt2tstr(60*60*24+3660,starttime)
  print (timestr)

  starttime = "1996:12:31:00:00:0.0"
  mhdt      = tstr2mhdt('1996:12:31:03:00:0.0',starttime)
  print (mhdt)

  starttime = "1998:01:01:00:00:0.0"
  timestr      = mhdt2tstr(1.6844600e+08,starttime)
  print (timestr)


if __name__=="__main__":
  main()




#  Jimmy's python modules
#  2005-12-27
#  Jimmy Raeder
#  UNH Space Science Center  J.Raeder@unh.edu

import locale

def read_tseries(file,t1,t2):
  f = open(file,'r'); l = f.read().split('\n'); f.close(); ll = []; ww = []
  #...... convert to float and pick out time

  for s in l:
    w = s.split(None)
#    print('w',w)
    if len(w) == 7:
      iy = locale.atof(w[0]); mo = locale.atof(w[1]); id = locale.atof(w[2])
      ih = locale.atof(w[3]); mi = locale.atof(w[4]); se = locale.atof(w[5])
      vv = locale.atof(w[6]); tt = epo1(iy,mo,id,ih,mi,se)
      if (tt >= t1 and tt <= t2):
        w[0]=iy; w[1]=mo; w[2]=id; w[3]=ih; w[4]=mi; w[5]=se; w[6]=tt; w.append(vv)
        ll.append(w)

  #...... transpose
  if ll == [] : return 0
  else: x = [[r[col] for r in ll] for col in range(len(ll[0]))]

#  x = [[r[col] for r in ll] for col in range(len(ll[0]))]

  return x

def epo1c(c):
  w=c.split(':')
  return(epo1(locale.atoi(w[0]),
              locale.atoi(w[1]),
              locale.atoi(w[2]),
              locale.atoi(w[3]),
              locale.atoi(w[4]),
              locale.atof(w[5])))

def epo1(iy,mo,id,ih,mi,se):
  if iy < 200 : iy = 1900 + iy
  i1=367*iy
  i2=int((mo+9)/12)
  i2=(i2+iy)*7
  i2=int(i2/4)
  i3=int((275*mo)/9)
  i4=100*iy+mo
  d1=(1.0)
  if (i4-190002.5) < 0.0 : d1=(-1.0)
  rr = ( i1 - i2 + i3 + id + 1721013.5 -0.5*d1 +0.5 )*86400.0
  rr = rr + 3600.0*ih + 60.0*mi +se
  return(rr)



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import pickle as pl
import io
from sys import argv
from read_mhd                import *
from pkg_mhdplot             import mhd_vpkg
from openggcm_info           import runinfo,mhdtime,mhdt2tstr
from matplotlib.colors       import LogNorm
from matplotlib.ticker       import MultipleLocator, FuncFormatter
from mpl_toolkits.axes_grid1 import make_axes_locatable

run=argv[1]
if run=='Msh_Nstep':external='easystore'
else:external='Elements'

x=np.linspace(0,20)

for i in range(1,7):
    a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_data_0.1.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
    a2=a[a.the==0]
    if argv[1]=='cs':
        b=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/sheath_data_ascii/cs%d.txt' % i,sep='\s+',header=None,names=['theta_1','theta_2','f_1','f_2','B','ni','Ti','V'],skiprows=1)
    else:
        b=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/sheath_data_ascii/cn%d.txt' % i,sep='\s+',header=None,names=['theta_1','theta_2','f_1','f_2','B','ni','Ti','V'],skiprows=1)
    b['phi1']=np.where(b.theta_1<=180,b.theta_1,b.theta_1-360)
    b['phi2']=np.where(b.theta_2<=180,b.theta_2,b.theta_2-360)
    b['phi']=(b.phi1+b.phi2)/2
    b['f']=(b.f_1+b.f_2)/2
#    b2=b[(-90<=b.phi) & (b.phi<=90)]
    b2=b

    df_list2=[]
    for phi in np.arange(-90+7.5,90,7.5):
        y=x*np.tan(phi*np.pi/180)

        b3=b2[(b2.phi<phi+.01) & (b2.phi>phi-.01)]
        df1 = pd.DataFrame([[np.nan] * len(b3.columns)], columns=b3.columns)
        b3 = df1.append(b3, ignore_index=True)
        b3.iloc[0].f=0
        b3.iloc[0].phi=phi
        b3=b3.append(pd.Series(),ignore_index=True)
        b3.iloc[-1].f=1
        b3.iloc[-1].phi=phi
        #print(b3.f)

        b4=b3[['f','B','ni','Ti','V']].set_index('f')
        b4.columns=['B_T','ni_T','Ti_T','V_T']

        df_list=[]
        for j in range(0,11):
            a_t=a2[(a2.phi<phi+7.5) & (a2.phi>phi-7.5)]
            a_t2=a_t[(a_t.m>j-.1) & (a_t.m<j+.1)]
            a_t3=a_t2.mean().to_frame().T
            a_t3.phi=phi
            a_t3['f']=j/10
            df_list.append(a_t3)
        a3=pd.concat(df_list,axis=0).set_index('f')

        a4=a3.reindex(a3.index.union(b3.f))
        #print(a3.index.union(b3.f))
        a5=a4.interpolate('index').reindex(index=b3.f)
        a5['B']=np.sqrt(a5.bx1**2+a5.by1**2+a5.bz1**2)
        a5['Ti']=72429.0*a5.pp1/a5.rr1/11600.
        a5['V']=np.sqrt(a5.vx1**2+a5.vy1**2+a5.vz1**2)
        a5['ni']=a5.rr1
        a6=a5[['rad','B','ni','Ti','V']]

        comp=pd.concat([a6,b4],axis=1)
        comp2=comp[['rad','B_T','B','ni_T','ni','Ti_T','Ti','V_T','V']]
        comp2['phi']=phi
        df_list2.append(comp2)

    comp3=pd.concat(df_list2,axis=0)
    comp3.to_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/SMILE_'+str(1200+1200*i).zfill(6),float_format='%.3f')



from setuptools import setup, find_packages
from pathlib import Path
DESCRIPTION = 'Simplified Earth magnetosheath model'

this_directory = Path(__file__).parent
LONG_DESCRIPTION =  (this_directory / "README.md").read_text()

setup(
    name='Mshpy',
    version='0.0.50',
    packages=find_packages(),
    package_data={'Mshpy': ['data/**/*']},
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    install_requires=[
        'sympy'
    ],
    include_package_data=True,
)


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.interpolate import NearestNDInterpolator as NNDI
from scipy.interpolate import LinearNDInterpolator as LNDI
import glob
import datetime

#For LNDI:
#np.save('test',f)
#f2=np.load('test.npy')
#f2.item()(x)

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

#filepath
external='easystore'
run='Msh_Nstep'

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = 180/np.pi*np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = 180/np.pi*np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def bsdf(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*(5/3-1)*(Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=Symbol('t',positive="True")
    x=xl*t
    y=yl*t
    z=zl*t
    sol=solve(a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44,t)
    dist=np.linalg.norm([np.array((xl*sole,yl*sole,zl*sole),dtype=np.float32) for sole in sol])
    return dist

def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R

def genf(fun,nxyz,f):
    outl=[]
    for i,j in zip(nxyz,f):
        if i[2][2]<0 or i[2][2]>1:
            out=np.nan
        else:
            out1=fun[i[0]](i[2])[0]
            out2=fun[i[1]](i[2])[0]
            if i[0]==i[1]:
                out=out1
            else:
                out=(out1*(1-j)+out2*j)/2
        outl.append(out)
    outl=np.array(outl)
    return outl

def OMNIread(f):

    BErr='9999.99'			# err of B field
    VErr='99999.9'			# err of Velocity
    NErr='999.99'				# err of Number density

    time,Bx,By,Bz,V,n,Pd,Ma=[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            V.append(float(w[7]))
            n.append(float(w[8]))
            Pd.append(float(w[9]))
            Ma.append(float(w[10]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    V=np.array(V)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'n':n,'Pd':Pd,'Ma':Ma,'B':B_sw})
    return omni

def SSCread(f):
    dateparse=lambda x,y: pd.datetime.strptime(x+' '+y,'%y/%m/%d %H:%M:%S')
    sat=pd.read_csv(f,header=None,sep='\s+',parse_dates={'datetime':[0,1]},date_parser=dateparse)
    sat.datetime=pd.to_datetime(sat.datetime)
    return sat

def main():
#    al=[]
    Bxfl=[]
    Byfl=[]
    Bzfl=[]
    nfl=[]
    Tfl=[]
    Vxfl=[]
    Vyfl=[]
    Vzfl=[]
    for i in range(1,7):
        a=pd.read_csv('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_data_0.1.'+str(1200+1200*i).zfill(6),header=None,sep='\s+',
        names=['the','phi','m','rad','x','y','z','bx1','by1','bz1','vx1','vy1','vz1','rr1','pp1','ex1','ey1','ez1','xjx1','xjy1','xjz1','resis1']).drop_duplicates()
        a['T1']=72429*a.pp1/a.rr1/11600
        a['f']=a.m/10
        Bxf=LNDI((a.the,a.phi,a.f),a.bx1.values)
        Byf=LNDI((a.the,a.phi,a.f),a.by1.values)
        Bzf=LNDI((a.the,a.phi,a.f),a.bz1.values)
        nf=LNDI((a.the,a.phi,a.f),a.rr1.values)
        Tf=LNDI((a.the,a.phi,a.f),a.T1.values)
        Vxf=LNDI((a.the,a.phi,a.f),a.vx1.values)
        Vyf=LNDI((a.the,a.phi,a.f),a.vy1.values)
        Vzf=LNDI((a.the,a.phi,a.f),a.vz1.values)
        Bxfl.append(Bxf)
        Byfl.append(Byf)
        Bzfl.append(Bzf)
        nfl.append(nf)
        Tfl.append(Tf)
        Vxfl.append(Vxf)
        Vyfl.append(Vyf)
        Vzfl.append(Vzf)
        print(i,'/6 Constructing functions from OpenGGCM data')

    #For LNDI:
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Bxfl',Bxfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Byfl',Byfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Bzfl',Bzfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/nfl',nfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Tfl',Tfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Vxfl',Vxfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Vyfl',Vyfl)
    np.save('/Volumnes/easystore/opengggcm_run/Msh_Nstep/Vzfl',Vzfl)
    #f2=np.load('test.npy')
    #f2[0](x)



#    return pts_df
    return


if __name__=="__main__":
    main()



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from sys import argv
from sympy.solvers import solve
from sympy import Symbol,nsolve
from sympy import sqrt,log,tanh
from scipy.interpolate import NearestNDInterpolator as NNDI
from scipy.interpolate import LinearNDInterpolator as LNDI
from scipy.interpolate import RegularGridInterpolator as RGI
from scipy.spatial.transform import Rotation as R
import glob
import datetime

#For LNDI:
#np.save('test',f)
#f2=np.load('test.npy')
#f2.item()(x)

# Bow Shock
a11=.45
a22=1
a33=.8
a12=.18
a14=46.6
a24=-2.2
a34=-.6
a44=-618
R0=11.8954

#filepath

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz,np.empty(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    #ptsnew[:,4] = np.arctan2(np.sqrt(xy), xyz[:,2]) # for elevation angle defined from Z-axis down
    ptsnew[:,4] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def bsdf(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=Symbol('t',positive="True")
    x=xl*t
    y=yl*t
    z=zl*t
    sol=solve(a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44,t)
    dist=np.linalg.norm([np.array((xl*sole,yl*sole,zl*sole),dtype=np.float32) for sole in sol])
    return dist

def bsdf2(N,V,Ma,B,xl,yl,zl):
    D=0.937*(0.846+0.042*B)
    f=91.55/(N*V**2)**(1/6)*(1+D*((5/3-1)*Ma**2+2)/((5/3+1)*(Ma**2-1)))/R0
    t=0
    sol0=a44
    sol=sol0
    while sol*sol0>0:
        t+=0.001
        x=xl*t
        y=yl*t
        z=zl*t
        sol=a11*(x/f)**2+a22*(y/f)**2+a33*(z/f)**2+a12*(x/f)*(y/f)+a14*(x/f)+a24*(y/f)+a34*(z/f)+a44
#        print(sol)
    dist=np.linalg.norm(np.array([xl*t,yl*t,zl*t]))
    return dist


def mpdf(Pd,Bz,xyz):
    yz=xyz[:,1]**2+xyz[:,2]**2
    theta=np.arctan2(np.sqrt(yz),xyz[:,0])
    r0=(10.22+1.29*np.tanh(0.184*(Bz+8.14)))*Pd**(-1/6.6)
    alpha=(0.58-0.007*Bz)*(1+0.024*np.log(Pd))
    R = r0*(2/(1+np.cos(theta)))**alpha
    return R

def OMNIreadnan(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,V,n,Pd,Ma=[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            V.append(float(w[7]))
            n.append(float(w[8]))
            Pd.append(float(w[9]))
            Ma.append(float(w[10]))
        else:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(np.nan)
            By.append(np.nan)
            Bz.append(np.nan)
            V.append(np.nan)
            n.append(np.nan)
            Pd.append(np.nan)
            Ma.append(np.nan)
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    V=np.array(V)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'n':n,'Pd':Pd,'Ma':Ma,'B':B_sw})
    return omni

def OMNIread2(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,vx,vy,vz,n,Pd,Ma,Mm=[],[],[],[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            vx.append(float(w[7]))
            vy.append(float(w[8]))
            vz.append(float(w[9]))
            n.append(float(w[10]))
            Pd.append(float(w[11]))
            Ma.append(float(w[12]))
            Mm.append(float(w[13]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    vx=np.array(vx)
    vy=np.array(vy)
    vz=np.array(vz)
    n=np.array(n)
    Pd=np.array(Pd)
    Ma=np.array(Ma)
    Mm=np.array(Mm)
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    V=np.sqrt(vx**2+vy**2+vz**2)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'vx':vx,'vy':vy,'vz':vz,'n':n,'Pd':Pd,'Ma':Ma,'Mm':Mm,'B':B_sw})
    return omni

def WINDread(f):

    BErr='9999.99'          # err of B field
    VErr='99999.9'          # err of Velocity
    NErr='999.99'               # err of Number density

    time,Bx,By,Bz,V,n,Pd,Ma=[],[],[],[],[],[],[],[]
    header1,header2=[],[]
    list=glob.glob(f)
    f0=open(list[0],'r')
    for line in f0:
        line=line.strip()
        w=line.split()
        if w[4]!=BErr and w[7]!=VErr and w[8]!=NErr:
            time.append(datetime.datetime(int(w[0]),1,1)+datetime.timedelta(days=int(w[1])-1,hours=int(w[2]),minutes=int(w[3])))
            Bx.append(float(w[4]))
            By.append(float(w[5]))
            Bz.append(float(w[6]))
            V.append(float(w[7]))
            n.append(float(w[8]))
    Bx=np.array(Bx)
    By=np.array(By)
    Bz=np.array(Bz)
    V=np.array(V)
    n=np.array(n)
    Pd=2e-6*n*V**2
    B_sw=np.sqrt(Bx**2+By**2+Bz**2)
    Ma=20*B_sw/np.sqrt(n)
    omni=pd.DataFrame({'datetime':time,'Bx':Bx,'By':By,'Bz':Bz,'V':V,'n':n,'Pd':Pd,'Ma':Ma,'B':B_sw})
    return omni

def SSCread(f):
    dateparse=lambda x,y: pd.datetime.strptime(x+' '+y,'%y/%m/%d %H:%M:%S')
    sat=pd.read_csv(f,header=None,sep='\s+',names=['d','t','x','y','z'],parse_dates={'datetime':[0,1]},date_parser=dateparse)
    sat.datetime=pd.to_datetime(sat.datetime)
    return sat

def genf(fun,nxyz,f,f2):
    outl=[]
    for i,j,k in zip(nxyz,f,f2):
        if i[2][2]<0.1 or i[2][2]>0.9:
            out=np.nan
        else:
            if k==-1:
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                out=(out1*(1-j)+out2*j)
            elif k==2:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                out=(out1*(1-j)+out2*j)
            else:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                out=(out1*(1-j)+out2*j)
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                outs=(out1*(1-j)+out2*j)
                out=k*out+(1-k)*outs
        outl.append(out)
    outl=np.array(outl)
    return outl

def genf_n(fun,nxyz,f,f2):
    outl=[]
    for i,j,k in zip(nxyz,f,f2):
        if i[2][2]<0 or i[2][2]>1:
            out=np.nan
        else:
            if k==-1:
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                if (i[0]==0 & i[1]==0) or (i[0]==6 & i[1]==6):out1=0
                out=(out1*(1-j)+out2*j)
            elif k==2:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                if (i[0]==0 & i[1]==0) or (i[0]==6 & i[1]==6):out1=0
                out=(out1*(1-j)+out2*j)
            else:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                if (i[0]==0 & i[1]==0) or (i[0]==6 & i[1]==6):out1=0
                outn=(out1*(1-j)+out2*j)
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                if (i[0]==0 & i[1]==0) or (i[0]==6 & i[1]==6):out1=0
                outs=(out1*(1-j)+out2*j)
                out=k*outn+(1-k)*outs
        outl.append(out)
    outl=np.array(outl)
    return outl

def genf_test(fun,nxyz,f,f2):
    outl=[]
    for i,j,k in zip(nxyz,f,f2):
        if i[2][2]<0 or i[2][2]>1:
            out=np.nan
        else:
            if k==-1:
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                out=(out1*(1-j)+out2*j)
            elif k==2:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                out=(out1*(1-j)+out2*j)
            else:
                out1=fun[i[0]](i[2])[0]
                out2=fun[i[1]](i[2])[0]
                out=(out1*(1-j)+out2*j)
                print(out1,out2,j,out)
                out1=fun[i[0]+6](i[2])[0]
                out2=fun[i[1]+6](i[2])[0]
                outs=(out1*(1-j)+out2*j)
                out=k*out+(1-k)*outs
        outl.append(out)
    outl=np.array(outl)
    return outl

def main(xyz,f_sw,fout,mpdo=0,bsdo=0,offsetf=0):
    mpdo=float(mpdo)
    bsdo=float(bsdo)
    if offsetf!=0:
        with open(offsetf,'w') as file:
            #print(offsetf)
            file.write(f'{mpdo} {bsdo}')



    if type(xyz)==str:
        xyz=SSCread(xyz)
        if (xyz.x<0).any():
            print('Warning: there is at least one time at which satellite was on nightside, which isn\'t supported by current version of model.')
            #print(xyz.x.max())

    omni=OMNIread2(f_sw)
    if len(omni)==1:
        omni=pd.concat([omni]*np.shape(xyz)[0])

    if type(xyz)!=np.ndarray:
        test=xyz.merge(omni,how='outer')[['datetime','x','y','z']].sort_values(by=['datetime']).set_index('datetime').interpolate('index')
        omni=omni.set_index('datetime').join(test,how='left')
        xyz_np=omni[['x','y','z']].to_numpy()
    else:
        xyz_np=xyz

    r=R.from_euler('z',-4,degrees=True)
    xyz_np=(r.as_matrix()@xyz_np.T).T

    print('Finding MP nodes')
    mpd=np.array(mpdf(omni.Pd,omni.Bz,xyz_np))
#    bsd=np.array([bsdf(omni.n[i],v_sw[i],Ma[i],B_sw[i],xyz[i])for i in range(np.shape(xyz)[0])])
    mpd+=mpdo

    print('Finding BS nodes')
    bsdfv=np.vectorize(bsdf2)
    bsd=np.array(bsdfv(omni.n,omni.V,omni.Ma,omni.B,xyz_np[:,0],xyz_np[:,1],xyz_np[:,2]))
    print(np.shape(mpd),np.shape(bsd))
    bsd+=bsdo

    pts=appendSpherical_np(xyz_np)
    print('theta range',np.nanmin(pts[:,4])*180/np.pi,np.nanmax(pts[:,4])*180/np.pi)
    print('phi range',np.nanmin(pts[:,5]*180/np.pi),np.nanmax(pts[:,5])*180/np.pi)

    f_msh=(pts[:,3]-mpd)/(bsd-mpd)
#    print(type(pts),type(f_msh))
    pts=np.hstack((pts,f_msh.reshape(pts.shape[0],1)))
    #print(pts)

    pts_df=pd.DataFrame(pts, columns=['x','y','z','r','theta','phi','f'])
    input=pts_df[['theta','phi','f']].to_numpy()

#    if (omni.n>35).any(): print('Warning: there are at least one time at which solar wind density goes above 35cm^-3, in which our model was not validated')
    if (omni.n>35).any(): print('Warning: there is at least one time at which solar wind density goes above 35cm^-3, in which our model was not validated')
    if (omni.B>15).any(): print('Warning: there is at least one time at which IMF magnitude goes above 15cm^-3, in which our model shows discrepancy with THEMIS data')
    conditions=[omni.n<=5,(5<omni.n) & (omni.n<10),(10<=omni.n) & (omni.n<15),(15<=omni.n) & (omni.n<20),(20<=omni.n) & (omni.n<25),(25<=omni.n) & (omni.n<30),(30<=omni.n) & (omni.n<35)]
    choices1=[0,0,1,2,3,4,5]
    choices2=[0,1,2,3,4,5,5]
    n1=np.select(conditions,choices1)
    n2=np.select(conditions,choices2)
    f=omni.n/5-(n1+1)  #factor for fine tuning of SW density
    f[f<0]+=1
    conditions2=[omni.Bz<=-4,(-4<omni.Bz) & (omni.Bz<4), 4<omni.Bz]
    choices3=[-1,(omni.Bz+4)/8,2]
    f2=np.select(conditions2,choices3) #factor for fine tuning of IMF
    inputn=list(zip(n1,n2,input))
    #print(inputn)

#    al=[]
    Bxfl=[]
    Byfl=[]
    Bzfl=[]
    nfl=[]
    Tfl=[]
    Vxfl=[]
    Vyfl=[]
    Vzfl=[]
    the=np.arange(-90,91,1)*np.pi/180
    phi=np.arange(-90,91,1)*np.pi/180
    af=np.arange(0,11,1)/10

    print('Mean Bz=',omni.Bz.mean())

    external='easystore'
    run='Msh_Nstep'

    for i in range(1,7):
        bx1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bx_%d.npy'%i)
        by1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_By_%d.npy'%i)
        bz1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bz_%d.npy'%i)
        rr1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_n_%d.npy'%i)
        pp1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_pp_%d.npy'%i)
        T1=72429*pp1/rr1/11600
        vx1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vx_%d.npy'%i)
        vy1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vy_%d.npy'%i)
        vz1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vz_%d.npy'%i)

        Bxf=RGI((the,phi,af),bx1,bounds_error=False)
        Byf=RGI((the,phi,af),by1,bounds_error=False)
        Bzf=RGI((the,phi,af),bz1,bounds_error=False)
        nf=RGI((the,phi,af),rr1,bounds_error=False)
        Tf=RGI((the,phi,af),T1,bounds_error=False)
        Vxf=RGI((the,phi,af),vx1,bounds_error=False)
        Vyf=RGI((the,phi,af),vy1,bounds_error=False)
        Vzf=RGI((the,phi,af),vz1,bounds_error=False)
        Bxfl.append(Bxf)
        Byfl.append(Byf)
        Bzfl.append(Bzf)
        nfl.append(nf)
        Tfl.append(Tf)
        Vxfl.append(Vxf)
        Vyfl.append(Vyf)
        Vzfl.append(Vzf)

    external='Elements'
    run='Msh_Nstep_cs'

    for i in range(1,7):
        bx1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bx_%d.npy'%i)
        by1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_By_%d.npy'%i)
        bz1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Bz_%d.npy'%i)
        rr1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_n_%d.npy'%i)
        pp1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_pp_%d.npy'%i)
        T1=72429*pp1/rr1/11600
        vx1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vx_%d.npy'%i)
        vy1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vy_%d.npy'%i)
        vz1=np.load('/Volumes/'+external+'/openggcm_run/'+run+'/modeling/Msh_Vz_%d.npy'%i)

        Bxf=RGI((the,phi,af),bx1,bounds_error=False)
        Byf=RGI((the,phi,af),by1,bounds_error=False)
        Bzf=RGI((the,phi,af),bz1,bounds_error=False)
        nf=RGI((the,phi,af),rr1,bounds_error=False)
        Tf=RGI((the,phi,af),T1,bounds_error=False)
        Vxf=RGI((the,phi,af),vx1,bounds_error=False)
        Vyf=RGI((the,phi,af),vy1,bounds_error=False)
        Vzf=RGI((the,phi,af),vz1,bounds_error=False)
        Bxfl.append(Bxf)
        Byfl.append(Byf)
        Bzfl.append(Bzf)
        nfl.append(nf)
        Tfl.append(Tf)
        Vxfl.append(Vxf)
        Vyfl.append(Vyf)
        Vzfl.append(Vzf)

#        print(i,'/6 Constructing functions from OpenGGCM data')

#    test=Bxfl[0](input[0])
#    print(input[0],test)
#    np.shape(test)


    Bx=genf(Bxfl,inputn,f,f2)
    By=genf(Byfl,inputn,f,f2)
    Bz=genf(Bzfl,inputn,f,f2)
    n=genf(nfl,inputn,f,f2)
    T=genf(Tfl,inputn,f,f2)
    Vx=genf(Vxfl,inputn,f,f2)
    Vy=genf(Vyfl,inputn,f,f2)
    Vz=genf(Vzfl,inputn,f,f2)
    V_0=np.vstack([Vx,Vy,Vz]).T
    V=(r.as_matrix().T@V_0.T).T

    print(len(omni),len(f))
    out=pd.DataFrame({'time':omni.index,'Bx':Bx,'By':By,'Bz':Bz,'n':n,'Tev':T,'Vx':V[:,0],'Vy':V[:,1],'Vz':V[:,2],'f':pts_df.f})
    out.to_csv(fout,float_format='%.3f',index=False)



#    return pts_df
    return Bx,By,Bz,n,T,Vx,Vy,Vz


if __name__=="__main__":
    f_sw="SW_cond_test.txt"
    test=open(f_sw,"w")
    test.write("2003 124 12 30    1.01   -1.65   -0.78  -414.8   -21.8     0.7   7.54  2.60  24.3  7.7\n")
    #Year DOY HR MN Bx By Bz Vx Vy Vz n Pd Ma Mm, as in OMNIweb
    test.close()

    #f_sw='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/omni.lst'
    xyz=np.array([[12,2,3],[10,2,3]])
    #Orbit data from the SSWweb can be used as xyz. (filepath)
    #Data would be interpolated based on OMNI time.
    #xyz='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/cl_orbit.txt'
    fout='Msh_test_out.txt'
    #fout='/Volumes/easystore/openggcm_run/Msh_Nstep/case_studies/2003May04_001/Msh_test_out.txt'
    main(xyz,f_sw,fout)



#!/usr/bin/python
# make swdata from WIND MFI and SWE data

import os
import sys
sys.path.append("/home/hconnor/svn/hkc-project/trunk/lib/")

import locale
import matplotlib.pyplot as plt
import numpy as np
from read_sc		import *
from jrmod01  		import *
from numpy		import arange,average
from openggcm_info 	import mhdtime,mhdt2hrmi


def WI_swdata(fi1,fi2,t1,t2,t3,t4,av,setbx):
  "create swdata, the OpenGGCM input from WIND MFI and SWE data"

## interpolate WIND data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','vth','rr','xgse','ygse','zgse', \
          'vxgse','vygse','vzgse','pp']
  fo   = [ 'wi.'+i for i in name ]

  lda1=ldaf(fi1)
  lda2=ldaf(fi2)

  date1,time1,bx,by,bz                   = read_WIMFI(fi1,lda1,setbx)
  date2,time2,th,np2,xl,yl,zl,vx,vy,vz,pp = read_WISWE(fi2,lda2)

  date = [date1,date1,date1,date2,date2,date2,date2,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time2,time2,time2,time2,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,th,np2,xl,yl,zl,vx,vy,vz,pp]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create swdata, the OpenGGCU input starting from t3
  name = ['bxgse','bygse','bzgse','vxgse','vygse','vzgse','rr','pp']
  fo   = [ 'wi.'+i for i in name ]
  create_swdata(fo,'swdata.wi',t1,t2,t3,t4)
  #print(np.shape(xl))
  print(np.average(np.array(xl).astype(float)))

  return


def AC_swdata(fi1,fi2,t1,t2,t3,t4,av,setbx):
  "create swdata, the OpenGGCM input from ACE MFI and SWE data"

## interpolate ACE data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','xgse','ygse','zgse', \
          'rr','pp','tk','vxgse','vygse','vzgse'        ]
  fo   = [ 'ac.'+i for i in name ]

  lda1=ldaf(fi1)
  lda2=ldaf(fi2)

  date1,time1,bx,by,bz,xl,yl,zl          = read_ACMFI(fi1,lda1,setbx)
  date2,time2,np,pp,tk,vx,vy,vz          = read_ACSWE(fi2,lda2)

  date = [date1,date1,date1,date1,date1,date1,date2,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time1,time1,time1,time2,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,xl,yl,zl,np,pp,tk,vx,vy,vz]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create swdata, the OpenGGCU input starting from t3
  name = ['bxgse','bygse','bzgse','vxgse','vygse','vzgse','rr','pp']
  fo   = [ 'ac.'+i for i in name ]
  create_swdata(fo,'swdata.ac',t1,t2,t3,t4)

  return


def omni_swdata(fi1,fi2,t1,t2,t3,t4,av,setbx):
  "create swdata, the OpenGGCM input from OMNI data"

## interpolate OMNI data every av seconds between t1 and t2

  name = ['bxgse','bygse','bzgse','tk','rr','xgse','ygse','zgse', \
          'vxgse','vygse','vzgse','pp']
  fo   = [ 'omni.'+i for i in name ]



#  print(lda1)
  lda1=0
  date1,time1,bx,by,bz,date2,time2,vx,vy,vz,nn,pp,tk,date3,time3,xl,yl,zl \
  = read_omni(fi1,lda1,setbx)



  date = [date1,date1,date1,date2,date2,date3,date3,date3,date2,date2,date2,date2]
  time = [time1,time1,time1,time2,time2,time3,time3,time3,time2,time2,time2,time2]
  vv   = [bx,by,bz,tk,nn,xl,yl,zl,vx,vy,vz,pp]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])



## create swdata, the OpenGGCM input starting from t3
  name = ['bxgse','bygse','bzgse','vxgse','vygse','vzgse','rr','pp']
  fo   = [ 'omni.'+i for i in name ]
  create_swdata(fo,'swdata.omni',t1,t2,t3,t4)

  return


def GE_swdata(fi1,fi2,lda1,lda2,t1,t2,t3,t4,av,setbx):
  "create swdata, the OpenGGCM input from Geotail data"

## interpolate OMNI data every av seconds between t1 and t2

  name = ['bxgse','bygse','bzgse','tk','rr','xgse','ygse','zgse', \
          'vxgse','vygse','vzgse','pp']
  fo   = [ 'ge.'+i for i in name ]

  date,time,bx,by,bz,vx,vy,vz,nn,pp,tk,xl,yl,zl = read_GE(fi1,lda1,setbx)

  date = [date,date,date,date,date,date,date,date,date,date,date,date]
  time = [time,time,time,time,time,time,time,time,time,time,time,time]
  vv   = [bx,by,bz,tk,nn,xl,yl,zl,vx,vy,vz,pp]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create swdata, the OpenGGCM input starting from t3
  name = ['bxgse','bygse','bzgse','vxgse','vygse','vzgse','rr','pp']
  fo   = [ 'ge.'+i for i in name ]
  create_swdata(fo,'swdata.ge',t1,t2,t3,t4)

  return

def TH_swdata(fi1,fi2,lda1,lda2,t1,t2,t3,t4,av,setbx,sc):
  "create swdata, the OpenGGCM input from ACE MFI and SWE data"

## interpolate THEMIS data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','rr','pp','tk',	\
          'vxgse','vygse','vzgse'        ]
  fo   = [ sc+'.'+i for i in name ]

  date1,time1,bx,by,bz           = read_THFGM(fi1,lda1,setbx)
  date2,time2,np,pp,tk,vx,vy,vz          = read_THESA(fi2,lda2)

  date = [date1,date1,date1,date2,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time2,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,np,pp,tk,vx,vy,vz]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create swdata, the OpenGGCU input starting from t3
  name = ['bxgse','bygse','bzgse','vxgse','vygse','vzgse','rr','pp']
  fo   = [ sc+'.'+i for i in name ]
  create_swdata(fo,'swdata.'+sc,t1,t2,t3,t4)
  return



def interpolate_data(fo,t1,t2,av,date,time,vv):
  "interpolate data every 'av' seconds between t1 and t2                        \
   fo   : output filename                                                       \
   t1   : start time of data                                                    \
   t2   : end time of data                                                      \
   av   : time interval for the interpolation                                   \
   date : date information                                                      \
   time : time information                                                      \
   vv   : data to be interpolated                                               "

  print("output filename: ",fo)
  ftmp="tmp."+fo
  f=open(ftmp,'w')
  for j in range(len(vv)):
    dd=date[j].split('-')
    tt=time[j].split(':')

    l1 = dd[2]+' '+dd[1]+' '+dd[0]+' '
    l2 = tt[0]+' '+tt[1]+' '+tt[2]+' '
    ll = l1+l2+' '+vv[j]+'\n'
    f.write(ll)
  f.close()

#  exe="$HOME/svn/hkc-project/trunk/src/swdata/tser_ave"
  exe="/Volumes/easystore/code/tser_ave"
  print(t1,t2,av,ftmp,fo)
  cmd=exe+" -t1 " + t1 + " -t2 " + t2 + " -tav " + av \
      +" -fillgaps < "+ftmp+" > " + fo

  print(cmd)

  os.system(cmd)
  os.remove(ftmp)

  return



def create_swdata(fi,fo,t1,t2,t3,t4):
  " create swdata, the OpenGGCM input, in the following format                  \
    T(min) bx(nT) by(nT) bz(nT) vx(km/s) vy(km/s) vz(km/s) N(cc) P(pP)          \
    ----------------------------------------------------------------------------\
    fi   : input data lists in the folloing order                               \
           [ bx,by,bz,vx,vy,vz,nn,pp ]                                          \
    fo   : output data                                                          \
    t1   : start time of solar wind data                                        \
    t2   : end   time of solar wind data                                        \
    t3   : start time of OpenGGCM run                                           \
    t4   : end   time of OpenGGCM run                                           \
    ----------------------------------------------------------------------------"

  vv  = []
  tt1 = epo1c(t1)
  tt2 = epo1c(t2)
  tt3 = epo1c(t3)
  tt4 = epo1c(t4)
  fmt = "%7.2f %8.4f %8.4f %8.4f %8.2f %8.2f %8.2f %7.2f %7.2f -1  0  0\n"



  for i in range(len(fi)):
    #print(fi,tt1,tt2)
    x=read_tseries(fi[i],tt1,tt2)
    #print(x)
    if i==0: vv.append(x[6])
    vv.append(x[7])



  f=open(fo,'w')

  for j in range(len(vv[0])):

   if vv[0][j] >=tt3 and vv[0][j]<=tt4:

     t=(vv[0][j]-tt3)/60.
     if t==0:
       ll = fmt%(-999.00,vv[1][j],vv[2][j],vv[3][j],vv[4][j],\
               vv[5][j],vv[6][j],vv[7][j],vv[8][j])
       f.write(ll)

     ll = fmt%(t,vv[1][j],vv[2][j],vv[3][j],vv[4][j], \
               vv[5][j],vv[6][j],vv[7][j],vv[8][j])
     f.write(ll)

  f.close()

  return


def read_swdata(fi):
  "read swdata"

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  tt=[];rr=[];pp=[]
  bx=[];by=[];bz=[]
  vx=[];vy=[];vz=[]

  for s in l:
    w=s.split(None)
    if len(w)>2:

      tt.append(locale.atof(w[0])*60.)

      bx.append(locale.atof(w[1]))
      by.append(locale.atof(w[2]))
      bz.append(locale.atof(w[3]))
      vx.append(locale.atof(w[4]))
      vy.append(locale.atof(w[5]))
      vz.append(locale.atof(w[6]))
      rr.append(locale.atof(w[7]))
      pp.append(locale.atof(w[8]))

  return tt,bx,by,bz,vx,vy,vz,rr,pp


def compare_swdata(time,sc1,sc2,tsc1,tsc2):
  "compare swdata from ACE and WIND 				\
   time    : start time of OpenGGCM RUN				\
   tac     : time shift of sc1 data to propagate SW propagation	\
   twi     : time shift of sc2 data to propagate SW propagation	"


  pm=1.672614e-27                 # proton mass

  fi='swdata.'+sc1
  tt,bx1,by1,bz1,vx1,vy1,vz1,rr1,pp1=read_swdata(fi)
  tt1=[ i + tsc1 for i in tt ]
# pp1=[ i*1e-3  for i in pp ]
  pdy1 = [ rr1[i]*pm*(vx1[i]**2+vy1[i]**2+vz1[i]**2)*1e21 for i in range(len(vx1)) ]
  bt1  = [ (bx1[i]**2+by1[i]**2+bz1[i]**2)**0.5 for i in range(len(bx1)) ]

  fi='swdata.'+sc2
  tt,bx2,by2,bz2,vx2,vy2,vz2,rr2,pp2=read_swdata(fi)
  tt2=[ i + tsc2 for i in tt ]
# pp2=[ i*1e-3  for i in pp ]
  pdy2 = [ rr2[i]*pm*(vx2[i]**2+vy2[i]**2+vz2[i]**2)*1e21 for i in range(len(vx2)) ]
  bt2  = [ (bx2[i]**2+by2[i]**2+bz2[i]**2)**0.5 for i in range(len(bx2)) ]

# print pp1,pp2
# print pdy1,pdy2
  vv1=[bx1,by1,bz1,bt1,vx1,vy1,vz1,rr1,pp1,pdy1]
  vv2=[bx2,by2,bz2,bt2,vx2,vy2,vz2,rr2,pp2,pdy2]


  vstr=[ 'Bx [nT]','By [nT]','Bz [nT]','Bt [nT]',		\
         'Vx [km/s]','Vy [km/s]','Vz [km/s]', 			\
         'N [/m3]','P [pPa]','Psw [nPa]'			]

  t0      =  mhdtime(time)
  title   ='SW data from '+sc1+'(blue) & '+sc2+'(green) on'+t0.date

  dtmaj   =  3600
  dtmin   =  600
  dtlab   =  3600*2
  xtick1  =  arange(0,tt[-1]+dtmaj,dtmaj)       # major tick location
  xtick2  =  arange(0,tt[-1]+dtmin,dtmin)       # minor tick location
  xlabel  =  mhdt2hrmi(xtick1,t0,dtlab)         # create tick label


  fig=plt.figure(figsize=(8,11))

  for i in range(len(vv1)) :
    ax1=plt.subplot(len(vv1),1,i+1)
    pl1=ax1.plot(tt1,vv1[i])
    pl2=ax1.plot(tt2,vv2[i])

    ax1.set_ylabel(vstr[i],fontsize=10)

    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)
    ax1.set_xticklabels(xlabel,fontsize=10)
    ax1.set_xlim   (0.,tt[-1])

    ax1.grid(axis='x')

    if i==0         : ax1.set_title (title,fontsize=12)
    if i==len(vv1)-1 : ax1.set_xlabel('UT',fontsize=10)

  fo='compare_swdata_'+t0.date+'.pdf'
  plt.savefig(fo)
  return


def plot_swdata(sc,time,dt=1) :
  "plot swdata									\
   sc    : satellite name, ac for ACE and wi for WIND				\
   time  : start time of OpenGGCM						\
   dt    : time interval for tick in hour, i.e., dt=4 makes ticks every 4 hours "

  pm=1.672614e-27                 # proton mass

  fi='swdata.'+sc
  tt,bx,by,bz,vx,vy,vz,rr,pp=read_swdata(fi)

  pdy = [ rr[i]*pm*(vx[i]**2+vy[i]**2+vz[i]**2)*1e21 for i in range(len(vx)) ]
  bt  = [ (bx[i]**2+by[i]**2+bz[i]**2)**0.5 for i in range(len(bx)) ]
  vt  = [ (vx[i]**2+vy[i]**2+vz[i]**2)**0.5 for i in range(len(vx)) ]
  rv  = [ rr[i]*vt[i]*1e5/1e8 for i in range(len(rr)) ]

# for  i in range(len(bx)):
#  if tt[i]> 7200 and tt[i] < 10800: print tt[i],bt[i],vx[i],pdy[i]

  vv=[bx,by,bz,bt,vx,vy,vz,rr,pp,pdy,rv]
  vstr=[ 'Bx [nT]','By [nT]','Bz [nT]','|B| [nT]',			\
         'Vx [km/s]','Vy [km/s]','Vz [km/s]', 				\
         'N [cm$^{-3}$]','Pth=nkT [pPa]','Pdy='+r'$\rho v^2$'+' [nPa]', \
         r'Flux=$\rho v$ [10$^8$ cm$^{-2}$s$^{-1}$]']

  t0      =  mhdtime(time)
  if   sc=='ac'  : title='SW data from ACE  on'+t0.date
  elif sc=='wi'  : title='SW data from WIND on '+t0.date
  elif sc=='ge'  : title='SW data from Geotail on '+t0.date
  elif sc=='omni': title='SW data from OMNI on '+t0.date
  else           : title='SW data from '+sc+' on '+t0.date


  dtmaj   =  3600*dt
  dtmin   =  600
  dtlab   =  3600*dt
  xtick1  =  arange(0,tt[-1]+dtmaj,dtmaj)   	# major tick location
  xtick2  =  arange(0,tt[-1]+dtmin,dtmin)   	# minor tick location
  xlabel  =  mhdt2hrmi(xtick1,t0,dtlab)         # create tick label

  fig=plt.figure(figsize=(8,11))


  for i in range(len(vv)) :
    ax1=plt.subplot(len(vv),1,i+1)
    plt.subplots_adjust(hspace = .001)
    ax1.tick_params(axis='both',which='major',labelsize=10)


    pl1=ax1.plot(tt,vv[i])

#   ax1.set_ylabel(vstr[i],fontsize=10)

    ax1.set_xticks (xtick1)
    ax1.set_xticks (xtick2,minor=True)
    ax1.set_xlim   (0.,tt[-1])
    ax1.grid       (axis='x')
    if i==len(vv)-1: ax1.set_xticklabels(xlabel)
    else           : ax1.set_xticklabels ([ '' for j in xlabel])

    plt.setp(ax1.get_yticklabels()[-1],visible=False)

    if i==0         : ax1.set_title (title,fontsize=12)
    if i==len(vv)-1 : ax1.set_xlabel('UT',fontsize=10)
    if i<=2         : ax1.axhline(y=0,ls=':',c='k')

    ax1.text(0.01,0.9,vstr[i],ha='left',va='top', \
             fontsize=10,transform=ax1.transAxes)

    if i==0         :
      ax1.text(0.99,0.1,'Ave_bx: %f'%average(bx),ha='right',va='bottom', \
               fontsize=10,transform=ax1.transAxes)
# plt.tight_layout()
  if bx[0]==bx[-1]: fo=fi+'_'+t0.date+'_constBz.pdf'
  else            : fo=fi+'_'+t0.date+'.pdf'
  plt.savefig(fo)
  return


def main(argv):

  sc=argv[0]			# satellite info
  f1=argv[1]           		#MFI data: date time bxgse bygse bzgse
  f2=argv[2]           		#SWE data: date time vth rr xgse ygse zgse vxgse vygse vzgse
  t1=argv[3]          		#starttime for interpolation
  t2=argv[4]            	#endtime   for interpolation
  t3=argv[5]           		#starttime for MHD simulation
  t4=argv[6]           		#endtime   for MHD simulation
  av=argv[7]      		#time interval of interpolation in sec
  dt=locale.atof(argv[8])	#time for major tick  in hr for plot

#  lda=argv[9].split(':')	#the line where data starts
 # lda1=locale.atoi(lda[0])-1
  #lda2=locale.atoi(lda[1])-1

  setbx=locale.atof(argv[9])  	#set up Bx value : 9999.99 for no change in Bx



#  print(lda,lda1,lda2)

### create swdata from WIND
  if sc=='wi'     : WI_swdata  (f1,f2,t1,t2,t3,t4,av,setbx)
  if sc=='ac'     : AC_swdata  (f1,f2,t1,t2,t3,t4,av,setbx)
  if sc=='ge'     : GE_swdata  (f1,f2,t1,t2,t3,t4,av,setbx)
  if sc=='omni'   : omni_swdata(f1,f2,t1,t2,t3,t4,av,setbx)
  if sc[:2]=='TH' : TH_swdata  (f1,f2,t1,t2,t3,t4,av,setbx,sc)

### plot swdata

  plot_swdata(sc,t3,dt)

# compare_swdata(t3,'wi','omni',-1700,0)
# plot_swdata('wi',t3)
# plot_swdata('ac',t3)
# compare_swdata(t3,0,0)

  return


if __name__=='__main__':
   main(sys.argv[1:])



#!/usr/bin/python
#
# get sc orbit file to obtain MHD values along spacecraft orbit


import os
import sys
HOME=os.getenv("HOME")
sys.path.append(HOME+"/svn/hkc-project/trunk/lib/")
sys.path.append(HOME+"/svn/hkc-project/trunk/src/swdata/")

import locale
#from jrmod01            import *
from numpy              import arange
from openggcm_info      import mhdtime,mhdt2hrmi
from swdata		import interpolate_data
import numpy as np

def ldaf(filename):
    with open(filename) as myFile:
        for num, line in enumerate(myFile, 1):
            if line.startswith('dd'):
                lda=num
#                print ('found at line:', num)
    return lda


def read_GEMGF(fi,vtype='string'):
  "read Geotail MGF data downloaded from SPDweb"

  Err = '-1.00000E+31'
  re  = 6.37814e3

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")


  date=[]
  time=[]
  bx=[];by=[];bz=[]
  xl=[];yl=[];zl=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err and w[4]!=Err and \
       w[5]!=Err and w[6]!=Err and w[7]!=Err                   :

      date.append(w[0])
      time.append(w[1])
      bx1=locale.format('%f',locale.atof(w[2])*0.1)
      by1=locale.format('%f',locale.atof(w[3])*0.1)
      bz1=locale.format('%f',locale.atof(w[4])*0.1)
      bx.append(bx1)
      by.append(by1)
      bz.append(bz1)
      x1=locale.format('%f',locale.atof(w[5])/re)
      y1=locale.format('%f',locale.atof(w[6])/re)
      z1=locale.format('%f',locale.atof(w[7])/re)
      xl.append(x1)
      yl.append(y1)
      zl.append(z1)

  if vtype=='real':
    bx = [locale.atof(i) for i in bx ]
    by = [locale.atof(i) for i in by ]
    bz = [locale.atof(i) for i in bz ]
    xl = [locale.atof(i) for i in xl  ]
    yl = [locale.atof(i) for i in yl  ]
    zl = [locale.atof(i) for i in zl  ]

  return date,time,bx,by,bz,xl,yl,zl

def read_GEH0CPI(fi,vtype='string'):
  "read Geotail CPI data"

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nsw=[];tk=[];teV=[];nhp=[];thp=[];php=[];
  vswx=[];vswy=[];vswz=[];vhpx=[];vhpy=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err  and w[4]!=Err  and  \
       w[5]!=Err and w[6]!=Err :

      date.append(w[0])
      time.append(w[1])
#     tsw.append(w[2])
      teV.append(locale.format('%f',locale.atof(w[2])/11600.))  # convert K to eV
      tk.append(w[2])
      nsw.append(w[3])
      vswx.append(w[4])
      vswy.append(w[5])
      vswz.append(w[6])

  if vtype=='real':
    nsw  = [locale.atof(i) for i in nsw ]
    tk   = [locale.atof(i) for i in tk  ]
    teV  = [locale.atof(i) for i in teV ]
    vswx = [locale.atof(i) for i in vswx ]
    vswy = [locale.atof(i) for i in vswy ]
    vswz = [locale.atof(i) for i in vswz ]

  return date,time,nsw,tk,teV,vswx,vswy,vswz

def read_GESWCPI(fi,vtype='string'):
  "read Geotail SW CPI data"

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)

  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nsw=[];tk=[];teV=[];nhp=[];thp=[];php=[];
  vswx=[];vswy=[];vswz=[];vhpx=[];vhpy=[];vsw=[];theta=[];phi=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err  and w[4]!=Err  and  \
       w[5]!=Err and w[6]!=Err :

      date.append(w[0])
      time.append(w[1])
#     tsw.append(w[2])
      teV.append(locale.format('%f',locale.atof(w[5])/11600.))  # convert K to eV
      tk.append(w[5])
      nsw.append(w[6])
      vsw.append(w[2])
      theta.append(w[3])
      phi.append(w[4])

  if vtype=='real':
    nsw  = [locale.atof(i) for i in nsw ]
    tk   = [locale.atof(i) for i in tk  ]
    teV  = [locale.atof(i) for i in teV ]
    vsw = [locale.atof(i) for i in vsw ]
    theta = [locale.atof(i) for i in theta ]
    phi = [locale.atof(i) for i in phi ]
    vswx = [v*np.sin(the*np.pi/180)*np.cos(phi*np.pi/180) for v,the,phi in zip(vsw,theta,phi)]
    vswy = [v*np.sin(the*np.pi/180)*np.sin(phi*np.pi/180) for v,the,phi in zip(vsw,theta,phi)]
    vswz = [v*np.cos(the*np.pi/180) for v,the,phi in zip(vsw,theta,phi)]
    print('vsw min',np.min(nsw))


  return date,time,nsw,tk,teV,vswx,vswy,vswz


def read_GEK0CPI(fi,vtype='string'):
  "read Geotail CPI data"

  Err = '-1.00000E+31'

  f=open(fi,'r')
  l=f.read().split('\n')
  f.close()

  lda=ldaf(fi)
  
  print("===============  CLEAN ALL UNNECESSARY INFORMATION  ===============")
  print("REMOVE ALL HEADERS BEFORE")
  print(l[lda])
  del   l[:lda]
  print("REMOVE ALL THE FOLLOWING TAILS ")
  print(l[-4:])
  del   l[-4:]
  print("-------------------------------------------------------------------")
  print("THE FIRST & LAST DATA LINES ARE")
  print(l[0])
  print(l[-1])
  print("===================================================================")

  date=[];time=[];nsw=[];tsw=[];nhp=[];thp=[];php=[];
  vswx=[];vswy=[];vswz=[];vhpx=[];vhpy=[]

  for s in l:
    w=s.split(None)
    if len(w)>2  and w[2]!=Err and w[3]!=Err  and w[4]!=Err  and  \
       w[5]!=Err and w[9]!=Err and w[10]!=Err and w[11]!=Err      :

      date.append(w[0])
      time.append(w[1])
      nsw.append(w[2])
#     tsw.append(locale.format('%f',locale.atof(w[3])*11600.))	# convert eV to K
      tsw.append(w[3])
      nhp.append(w[4])
#     thp.append(locale.format('%f',locale.atof(w[5])*11600.))  # convert eV to K
      thp.append(w[5])
      php.append(locale.format('%f',locale.atof(w[6])*1e9))	# pressure in  nP
      vswx.append(w[7])
      vswy.append(w[8])
      vswz.append(w[9])
      vhpx.append(w[10])
      vhpy.append(w[11])

  if vtype=='real':
    nsw  = [locale.atof(i) for i in nsw ]
    tsw  = [locale.atof(i) for i in tsw ]	# convert eV to K
    nhp  = [locale.atof(i) for i in nhp ]
    thp  = [locale.atof(i) for i in thp ]	# convert eV to K
    php  = [locale.atof(i) for i in php ]
    vswx = [locale.atof(i) for i in vswx ]
    vswy = [locale.atof(i) for i in vswy ]
    vswz = [locale.atof(i) for i in vswz ]
    vhpx = [locale.atof(i) for i in vhpx ]
    vhpy = [locale.atof(i) for i in vhpy ]

  return date,time,nsw,tsw,nhp,thp,php,vswx,vswy,vswz,vhpx,vhpy


def create_GEH0data(sc,fi1,fi2,lda1,lda2,t1,t2,t3,t4,av):
  "interpolate Geotail MGF and CPI data"

## interpolate Geotail data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','xgse','ygse','zgse', \
          'nsw','tsw','vswx','vswy','vswz' ]
  fo   = [ sc+'.'+i for i in name ]

  date1,time1,bx,by,bz,xl,yl,zl                            = read_GEMGF(fi1,lda1)
  date2,time2,nsw,tsw,vswx,vswy,vswz                       = read_GEH0CPI(fi2,lda2)

  date = [date1,date1,date1,date1,date1,date1,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time1,time1,time1,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,xl,yl,zl,nsw,tsw,vswx,vswy,vswz]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create orbit, the OpenGGCU input starting from t3
  name = ['xgse','ygse','zgse' ]
  fo   = [ sc+'.'+i for i in name ]
  create_GEorbit(fo,sc+'.GSEorbit',t1,t2,t3,t4)
  return


def create_GEK0data(sc,fi1,fi2,lda1,lda2,t1,t2,t3,t4,av):
  "interpolate Geotail MGF and CPI data"

## interpolate Geotail data every av seconds between t1 and t2
  name = ['bxgse','bygse','bzgse','xgse','ygse','zgse', \
          'nsw','tsw','nhp','thp','php','vswx','vswy','vswz','vhpx','vhpy'  ]
  fo   = [ sc+'.'+i for i in name ]

  date1,time1,bx,by,bz,xl,yl,zl                            = read_GEMGF(fi1,lda1)
  date2,time2,nsw,tsw,nhp,thp,php,vswx,vswy,vswz,vhpx,vhpy = read_GECPI(fi2,lda2)

  date = [date1,date1,date1,date1,date1,date1,date2,date2,date2,date2,date2,date2]
  time = [time1,time1,time1,time1,time1,time1,time2,time2,time2,time2,time2,time2]
  vv   = [bx,by,bz,xl,yl,zl,nsw,tsw,nhp,thp,php,vswx,vswy,vswz,vhpx,vhpy]

  for i in range(len(date)) :
    interpolate_data(fo[i],t1,t2,av,date[i],time[i],vv[i])

## create orbit, the OpenGGCU input starting from t3
  name = ['xgse','ygse','zgse' ]
  fo   = [ sc+'.'+i for i in name ]
  create_GEorbit(fo,sc+'.GSEorbit',t1,t2,t3,t4)
  return


def create_GEorbit(fi,fo,t1,t2,t3,t4):
  " create swdata, the OpenGGCM input, in the following format                  \
    T(min) bx(nT) by(nT) bz(nT) vx(km/s) vy(km/s) vz(km/s) N(cc) P(pP)          \
    ----------------------------------------------------------------------------\
    fi   : input data lists in the folloing order                               \
           [ bx,by,bz,vx,vy,vz,nn,pp ]                                          \
    fo   : output data                                                          \
    t1   : start time of solar wind data                                        \
    t2   : end   time of solar wind data                                        \
    t3   : start time of OpenGGCM run                                           \
    t4   : end   time of OpenGGCM run                                           \
    ----------------------------------------------------------------------------"

  vv  = []
  tt1 = epo1c(t1)
  tt2 = epo1c(t2)
  tt3 = epo1c(t3)
  tt4 = epo1c(t4)
  fmt = "%10.2f %8.4f %8.4f %8.4f\n"

  for i in range(len(fi)):
    x=read_tseries(fi[i],tt1,tt2)
    if i==0: vv.append(x[6])
    vv.append(x[7])

  f=open(fo,'w')
  for j in range(len(vv[0])):
   if vv[0][j] >=tt3 and vv[0][j]<=tt4:
     t=(vv[0][j]-tt3)
     ll = fmt%(t,vv[1][j],vv[2][j],vv[3][j])
     f.write(ll)
  f.close()
  return


def main(argv):

  sc=argv[0]                    # satellite info
  f1=argv[1]                    #MFI data: date time bxgse bygse bzgse
  f2=argv[2]                    #SWE data: date time vth rr xgse ygse zgse vxgse vygse vzgse
  t1=argv[3]                    #starttime for interpolation
  t2=argv[4]                    #endtime   for interpolation
  t3=argv[5]                    #starttime for MHD simulation
  t4=argv[6]                    #endtime   for MHD simulation
  av=argv[7]                    #time interval of interpolation in sec

  lda=argv[8].split(':')        #the line where data starts
  lda1=locale.atoi(lda[0])-1
  lda2=locale.atoi(lda[1])-1

  print(lda,lda1,lda2)

  create_GEdata(sc,f1,f2,lda1,lda2,t1,t2,t3,t4,av)


  return


if __name__=='__main__':
   main(sys.argv[1:])



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import interp1d


for i in range(0,7):
    a=pd.read_csv('modeling/xz_cusp_cn_n.'+str(1200*i).zfill(6),header=None,sep='\s+',skiprows=1)
    
